@@ -1,44 +1,47 @@
-;;;
-;;;      elmo-archive.el -- Multi-Archiver Interface for ELMO.
-;;;
-;;;             Copyright (C) 1998 OKUNISHI Fujikazu
-;;;             Copyright (C) 1998 TERANISHI Yuuichi
-;;;
-;;; Author:  OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
-;;; Created: Sep 13, 1998
-;;; Revised: Dec 15, 1998
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
+;;; elmo-archive.el -- Archive folder of ELMO.
 
-;;; TODO:
-;;; [ܥ] append-msgs() ߤʤ multi-refile Բġˡ
-;;; archive <=> archive, archive => localdir ǽʤ褦 copy-msgs() 
-;;; ܤ˽񤯡
-;;; Info-Zip ѥȤѤܸ측OS/2 ѡˡ
+;; Copyright 1998,1999,2000 OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
+;;                          Yuuichi Teranishi <teranisi@gohome.org>
 
+;; Author:  OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
+;; Keywords: mail, net news
+;; Created: Sep 13, 1998
+;; Revised: Dec 15, 1998
+
+;; This file is part of ELMO (Elisp Library for Message Orchestration).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
+;; TODO:
+;; [$B%\%=(B] append-msgs() $B$,M_$7$$!J$1$I(B multi-refile $BIT2D!K!#(B
+;; Info-Zip $B@lMQ%(!<%8%'%s%H$rMQ$$$?F|K\8l8!:w!J(BOS/2 $B@lMQ!K!#(B
 
 ;;; Code:
+;; 
+
 (require 'elmo-msgdb)
 (require 'emu)
 (require 'std11)
 (eval-when-compile (require 'elmo-localdir))
 
 ;;; Const
-(defconst elmo-archive-version "v0.17 [981215/alpha]")
+(defconst elmo-archive-version "v0.18 [990729/alpha]")
 
 ;;; User vars.
 (defvar elmo-archive-lha-dos-compatible
@@ -102,8 +105,7 @@
 ;    (arj . ".arj")
      (rar . ".rar")
      (tar . ".tar")
-     (tgz . ".tar.gz")
-     ))
+     (tgz . ".tar.gz")))
 
 ;;; lha
 (defvar elmo-archive-lha-method-alist
@@ -122,8 +124,7 @@
       (rm  . ("lha" "d"))
       (ls  . ("lha" "l"))
       (cat . ("lha" "pq"))
-      (ext . ("lha" "x"))
-      )))
+      (ext . ("lha" "x")))))
 
 ;;; info-zip/unzip
 (defvar elmo-archive-zip-method-alist
@@ -136,8 +137,7 @@
     (ls       . ("unzip" "-lq"))
     (cat      . ("unzip" "-pq"))
     (ext      . ("unzip"))
-    (cat-headers . ("izwlagent" "--cat"))
-    ))
+    (cat-headers . ("izwlagent" "--cat"))))
 
 ;;; zoo
 (defvar elmo-archive-zoo-method-alist
@@ -148,8 +148,7 @@
     (rm       . ("zoo" "Dq"))
     (ls       . ("zoo" "l"))  ; normal
     (cat      . ("zoo" "xpq"))
-    (ext      . ("zoo" "xq"))
-    ))
+    (ext      . ("zoo" "xq"))))
 
 ;;; rar
 (defvar elmo-archive-rar-method-alist
@@ -158,18 +157,19 @@
     (rm       . ("rar" "d"))
     (ls       . ("rar" "v"))
     (cat      . ("rar" "p" "-inul"))
-    (ext      . ("rar" "x"))
-    ))
+    (ext      . ("rar" "x"))))
 
 ;;; GNU tar (*.tar)
 (defvar elmo-archive-tar-method-alist
   (if elmo-archive-lha-dos-compatible
       '((ls   . ("gtar" "-tf"))
 	(cat  . ("gtar" "--posix Oxf"))
+	(ext  . ("gtar" "-xf"))
 	;;(rm   . ("gtar" "--posix" "--delete" "-f")) ;; well not work
 	)
   '((ls    . ("gtar" "-tf"))
     (cat   . ("gtar" "-Oxf"))
+    (ext   . ("gtar" "-xf"))
     ;;(rm    . ("gtar" "--delete" "-f")) ;; well not work
     )))
 
@@ -181,6 +181,7 @@
     ;;(rm       . elmo-archive-tgz-rm-func)
     (cp         . elmo-archive-tgz-cp-func)
     (mv         . elmo-archive-tgz-mv-func)
+    (ext        . ("gtar" "-zxf"))
     ;; tgz special method
     (decompress . ("gzip" "-d"))
     (compress   . ("gzip"))
@@ -196,8 +197,7 @@
     elmo-archive-tgz-method-alist
 ;   elmo-archive-arc-method-alist
 ;   elmo-archive-arj-method-alist
-    elmo-archive-rar-method-alist
-    ))
+    elmo-archive-rar-method-alist))
 
 ;;; Internal vars.
 (defvar elmo-archive-method-alist nil)
@@ -226,8 +226,7 @@
     (funcall method args output))
    (t
     (elmo-archive-call-process
-     (car method) (append (cdr method) args) output)
-    )))
+     (car method) (append (cdr method) args) output))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Scan Folder
@@ -280,7 +279,7 @@ TYPE specifies the archiver's symbol."
 
 (defsubst elmo-archive-get-archive-directory (name)
   ;; allow fullpath. return format is "/foo/bar/".
-  (if (string-match "^[/~].*\\|^[A-Z]:" name)
+  (if (file-name-absolute-p name)
       (if (find-file-name-handler name 'copy-file)
 	  name
 	(expand-file-name name))
@@ -314,7 +313,7 @@ TYPE specifies the archiver's symbol."
 	      (if (file-directory-p dbdir)
 		  (); ok.
 		(if (file-exists-p dbdir)
-		    (error "File %s already exists." dbdir)
+		    (error "File %s already exists" dbdir)
 		  (elmo-make-directory dbdir)))
 	      (if (not (file-exists-p filename))
 		  (copy-file
@@ -331,6 +330,9 @@ TYPE specifies the archiver's symbol."
   (file-exists-p
    (elmo-archive-get-archive-name (nth 1 spec) (nth 2 spec) spec)))
 
+(defun elmo-archive-folder-creatable-p (spec)
+  t)
+
 (defun elmo-archive-create-folder (spec)
   (let* ((dir (directory-file-name ;; remove tail slash.
 	       (elmo-archive-get-archive-directory (nth 1 spec))))
@@ -341,7 +343,7 @@ TYPE specifies the archiver's symbol."
     (cond ((and (file-exists-p dir)
 		(not (file-directory-p dir)))
            ;; file exists
-           (error "Create folder failed. File \"%s\" exists." dir))
+           (error "Create folder failed; File \"%s\" exists" dir))
           ((file-directory-p dir)
            (if (file-exists-p arc)
                t  ; return value
@@ -361,13 +363,13 @@ TYPE specifies the archiver's symbol."
 	   (args (list archive dummy)))
       (when (null method)
 	(ding)
-	(error "WARNING: read-only mode: %s (method undefined)." type))
+	(error "WARNING: read-only mode: %s (method undefined)" type))
       (cond
        ((file-directory-p tmp-dir)
 	()) ;nop
        ((file-exists-p tmp-dir)
 	;; file exists
-	(error "Create directory failed. File \"%s\" exists." tmp-dir))
+	(error "Create directory failed; File \"%s\" exists" tmp-dir))
        (t
 	(elmo-make-directory tmp-dir)))
       (elmo-bind-directory
@@ -379,6 +381,28 @@ TYPE specifies the archiver's symbol."
 	     (delete-file dummy)))
        ))))
 
+(defun elmo-archive-delete-folder (spec)
+  (let* ((arc (elmo-archive-get-archive-name (nth 1 spec) (nth 2 spec))))
+    (if (not (file-exists-p arc))
+	(error "no such file: %s" arc)
+      (delete-file arc)
+      t)))
+
+(defun elmo-archive-rename-folder (old-spec new-spec)
+  (let* ((old-arc (elmo-archive-get-archive-name
+		   (nth 1 old-spec) (nth 2 old-spec)))
+	 (new-arc (elmo-archive-get-archive-name
+		   (nth 1 new-spec) (nth 2 new-spec))))
+    (unless (and (eq (nth 2 old-spec) (nth 2 new-spec))
+		 (equal (nth 3 old-spec) (nth 3 new-spec)))
+      (error "not same archive type and prefix"))
+    (if (not (file-exists-p old-arc))
+	(error "no such file: %s" old-arc)
+      (if (file-exists-p new-arc)
+	  (error "already exists: %s" new-arc)
+	(rename-file old-arc new-arc)
+	t))))
+
 (defun elmo-archive-list-folders (spec &optional hierarchy)
   (let ((folder (concat "$" (nth 1 spec)))
 	(elmo-localdir-folder-path elmo-archive-folder-path))
@@ -434,9 +458,9 @@ TYPE specifies the archiver's symbol."
 	(and
 	 (as-binary-process
 	  (elmo-archive-call-method method args t))
-	 (elmo-delete-cr-get-content-type outbuf))))))
+	 (elmo-delete-cr-get-content-type))))))
 
-(defun elmo-archive-append-msg (spec string &optional msg) ;;; verrrrrry slow!!
+(defun elmo-archive-append-msg (spec string &optional msg no-see) ;;; verrrrrry slow!!
   (let* ((type (nth 2 spec))
 	 (prefix (nth 3 spec))
 	 (arc (elmo-archive-get-archive-name (nth 1 spec) type))
@@ -449,7 +473,7 @@ TYPE specifies the archiver's symbol."
 	 newfile)
     (when (null method)
       (ding)
-      (error "WARNING: read-only mode: %s (method undefined)." type))
+      (error "WARNING: read-only mode: %s (method undefined)" type))
     (save-excursion
       (set-buffer tmp-buffer)
       (erase-buffer)
@@ -472,18 +496,17 @@ TYPE specifies the archiver's symbol."
 	     nil))
 	(kill-buffer tmp-buffer)))))
 
-;;; localdir -> archive
-(defun elmo-archive-copy-msgs (src-spec msgs dst-spec expunge-it
-					&optional same-number) ;;xx expunge-it
-  (let* ((mlist msgs)
-	 (dst-type (nth 2 dst-spec))
+;;; (localdir, maildir, localnews, archive) -> archive
+(defun elmo-archive-copy-msgs (dst-spec msgs src-spec
+					&optional loc-alist same-number)
+  (let* ((dst-type (nth 2 dst-spec))
 	 (arc (elmo-archive-get-archive-name (nth 1 dst-spec) dst-type))
 	 (prefix (nth 3 dst-spec))
 	 (p-method (elmo-archive-get-method dst-type 'mv-pipe))
 	 (n-method (elmo-archive-get-method dst-type 'mv))
 	 (new (unless same-number
 		(1+ (car (elmo-archive-max-of-folder dst-spec)))))
-	 (src-dir (expand-file-name (nth 1 src-spec) elmo-localdir-folder-path))
+	 (src-dir (elmo-localdir-get-folder-directory src-spec))
 	 (tmp-dir
 	  (file-name-as-directory (elmo-msgdb-expand-path nil dst-spec)))
 	 (do-link t)
@@ -492,8 +515,9 @@ TYPE specifies the archiver's symbol."
       (elmo-archive-create-folder dst-spec))
     (when (null (or p-method n-method))
       (ding)
-      (error "WARNING: read-only mode: %s (method undefined)." dst-type))
+      (error "WARNING: read-only mode: %s (method undefined)" dst-type))
     (when (and same-number
+	       (not (eq (car src-spec) 'maildir))
 	       (string-match (concat prefix "$") src-dir)
 	       (or
 		(elmo-archive-get-method dst-type 'cp-pipe)
@@ -503,31 +527,24 @@ TYPE specifies the archiver's symbol."
 	    n-method (elmo-archive-get-method dst-type 'cp))
       (setq tmp-msgs (mapcar '(lambda (x) 
 				(elmo-concat-path prefix (int-to-string x)))
-			     mlist))
+			     msgs))
       (setq do-link nil))
     (when do-link
       (let ((tmp-dir (expand-file-name prefix tmp-dir)))
 	(when (not (file-directory-p tmp-dir))
 	  (elmo-make-directory (directory-file-name tmp-dir))))
-      (while mlist
+      (while msgs
 	(setq newfile (elmo-concat-path prefix (int-to-string
 						(if same-number
-						    (car mlist)
+						    (car msgs)
 						  new))))
 	(setq tmp-msgs (nconc tmp-msgs (list newfile)))
-	(condition-case err
-	    (elmo-add-name-to-file
-	     ;; src file
-	     (setq src (expand-file-name (int-to-string (car mlist))
-					 src-dir))
-	     ;; tmp file
-	     (setq tmp (expand-file-name newfile tmp-dir))
-	     t)
-	  (error (if (elmo-cross-device-link-error-p err)
-		     ;; cross device... copy it.
-		     (copy-file src tmp t)
-		   (error "copy file failed."))))
-	(setq mlist (cdr mlist))
+	(elmo-copy-file
+	 ;; src file
+	 (elmo-call-func src-spec "get-msg-filename" (car msgs) loc-alist)
+	 ;; tmp file
+	 (expand-file-name newfile tmp-dir))
+	(setq msgs (cdr msgs))
 	(unless same-number (setq new (1+ new)))))
     (save-excursion
       (elmo-bind-directory
@@ -546,86 +563,52 @@ TYPE specifies the archiver's symbol."
 	   (elmo-archive-exec-msgs-subr2
 	    n-prog (append n-prog-arg (list arc)) tmp-msgs (length arc)))))))))
 
-;; begin: added by simm, Mar 31, 1999
-;;; maildir -> archive
-(defun elmo-archive-copy-msgs-from-maildir (src-spec
-					    msgs dst-spec 
-					    expunge-it loc-alist
-					    &optional same-number) ;;xx expunge-it
-  (let* ((mlist msgs)
-	 (dst-type (nth 2 dst-spec))
-	 (arc (elmo-archive-get-archive-name (nth 1 dst-spec) dst-type))
-	 (prefix (nth 3 dst-spec))
-	 (p-method (elmo-archive-get-method dst-type 'mv-pipe))
-	 (n-method (elmo-archive-get-method dst-type 'mv))
-	 (new (unless same-number
-		(1+ (car (elmo-archive-max-of-folder dst-spec)))))
-	 ;;(src-dir (expand-file-name (nth 1 src-spec) elmo-localdir-folder-path))
-	 (src-dir (elmo-localdir-get-folder-directory src-spec))
+;;; archive -> (localdir, localnews, archive)
+(defun elmo-archive-copy-msgs-froms (dst-spec msgs src-spec
+					      &optional loc-alist same-number)
+  (let* ((src-type (nth 2 src-spec))
+	 (arc (elmo-archive-get-archive-name (nth 1 src-spec) src-type))
+	 (prefix (nth 3 src-spec))
+	 (p-method (elmo-archive-get-method src-type 'ext-pipe))
+	 (n-method (elmo-archive-get-method src-type 'ext))
 	 (tmp-dir
-	  (file-name-as-directory (elmo-msgdb-expand-path nil dst-spec)))
-	 (do-link t)
-	 src tmp newfile tmp-msgs)
-    (when (not (elmo-archive-folder-exists-p dst-spec))
-      (elmo-archive-create-folder dst-spec))
-    (when (null (or p-method n-method))
-      (ding)
-      (error "WARNING: read-only mode: %s (method undefined)." dst-type))
-    (when (and same-number
-	       (string-match (concat prefix "$") src-dir)
-	       (or
-		(elmo-archive-get-method dst-type 'cp-pipe)
-		(elmo-archive-get-method dst-type 'cp)))
-      (setq tmp-dir (substring src-dir 0 (match-beginning 0)))
-      (setq p-method (elmo-archive-get-method dst-type 'cp-pipe)
-	    n-method (elmo-archive-get-method dst-type 'cp))
-      (setq tmp-msgs (mapcar '(lambda (x) 
-				(elmo-concat-path prefix (int-to-string x)))
-			     mlist))
-      (setq do-link nil))
-    (when do-link
-      (let ((tmp-dir (expand-file-name prefix tmp-dir)))
-	(when (not (file-directory-p tmp-dir))
-	  (elmo-make-directory (directory-file-name tmp-dir))))
-      (while mlist
-	(setq newfile (elmo-concat-path prefix (int-to-string
-						(if same-number
-						    (car mlist)
-						  new))))
-	(setq tmp-msgs (nconc tmp-msgs (list newfile)))
-	(condition-case err
-	    (elmo-add-name-to-file
-	     ;; src file
-	     ;;(setq src (expand-file-name (int-to-string (car mlist))
-	     ;;				 src-dir))
-	     (setq src (elmo-maildir-number-to-filename
-			src-dir (car mlist) loc-alist))
-	     ;; tmp file
-	     (setq tmp (expand-file-name newfile tmp-dir))
-	     t)
-	  (error (if (elmo-cross-device-link-error-p err)
-		     ;; cross device... copy it.
-		     (copy-file src tmp t)
-		   (error "copy file failed."))))
-	(setq mlist (cdr mlist))
-	(unless same-number (setq new (1+ new)))))
-    (save-excursion
+	  (file-name-as-directory (elmo-msgdb-expand-path nil src-spec)))
+	 (tmp-msgs (mapcar '(lambda (x) (elmo-concat-path
+					 prefix
+					 (int-to-string x)))
+			   msgs))
+	 result)
+    (unwind-protect
+	(setq result
+	      (and
+	       ;; extract messages
+	       (save-excursion
+		 (elmo-bind-directory
+		  tmp-dir
+		  (cond
+		   ((functionp n-method)
+		    (funcall n-method (cons arc tmp-msgs)))
+		   (p-method
+		    (let ((p-prog (car p-method))
+			  (p-prog-arg (cdr p-method)))
+		      (elmo-archive-exec-msgs-subr1
+		       p-prog (append p-prog-arg (list arc)) tmp-msgs)))
+		   (t
+		    (let ((n-prog (car n-method))
+			  (n-prog-arg (cdr n-method)))
+		      (elmo-archive-exec-msgs-subr2
+		       n-prog (append n-prog-arg (list arc)) tmp-msgs (length arc)))))))
+	       ;; call elmo-*-copy-msgs of destination folder
+	       (elmo-call-func dst-spec "copy-msgs"
+			       msgs src-spec loc-alist same-number)))
+      ;; clean up tmp-dir
       (elmo-bind-directory
        tmp-dir
-       (cond
-	((functionp n-method)
-	 (funcall n-method (cons arc tmp-msgs)))
-	(p-method
-	 (let ((p-prog (car p-method))
-	       (p-prog-arg (cdr p-method)))
-	   (elmo-archive-exec-msgs-subr1
-	    p-prog (append p-prog-arg (list arc)) tmp-msgs)))
-	(t
-	 (let ((n-prog (car n-method))
-	       (n-prog-arg (cdr n-method)))
-	   (elmo-archive-exec-msgs-subr2
-	    n-prog (append n-prog-arg (list arc)) tmp-msgs (length arc)))))))))
-;; end: added by simm, Mar 31, 1999
+       (while tmp-msgs
+	 (if (file-exists-p (car tmp-msgs))
+	     (delete-file (car tmp-msgs)))
+	 (setq tmp-msgs (cdr tmp-msgs))))
+      result)))
 
 (defun elmo-archive-delete-msgs (spec msgs)
   (save-excursion
@@ -652,7 +635,7 @@ TYPE specifies the archiver's symbol."
 		n-prog (append n-prog-arg (list arc)) msgs (length arc))))
 	    (t
 	     (ding)
-	     (error "WARNING: not delete: %s (method undefined)." type))) )))
+	     (error "WARNING: not delete: %s (method undefined)" type))) )))
 
 (defun elmo-archive-exec-msgs-subr1 (prog args msgs)
   (let ((buf (get-buffer-create " *ELMO ARCHIVE exec*")))
@@ -721,18 +704,18 @@ TYPE specifies the archiver's symbol."
 	 )
     (when (null (and decompress compress exec))
       (ding)
-      (error "WARNING: special method undefined: %s of %s."
+      (error "WARNING: special method undefined: %s of %s"
 	     (or (if (null decompress) 'decompress)
 		 (if (null compress) 'compress)
 		 (if (null exec) exec-type))
 	     'tgz))
     (unless tar-suffix
       (ding)
-      (error "WARNING: `tar' suffix undefined."))
+      (error "WARNING: `tar' suffix undefined"))
     (if (string-match (concat (regexp-quote suffix) "$") arc)
 	(setq arc-tar
 	      (concat (substring arc 0 (match-beginning 0)) tar-suffix))
-      (error "%s: not match suffix [%s]." arc suffix))
+      (error "%s: not match suffix [%s]" arc suffix))
     (and
      ;; decompress
      (elmo-archive-call-process
@@ -751,8 +734,7 @@ TYPE specifies the archiver's symbol."
 	  (if (file-exists-p (car tmp-msgs))
 	      (delete-file (car tmp-msgs)))
 	  (setq tmp-msgs (cdr tmp-msgs))))
-    ret-val
-    ))
+    ret-val))
 
 (defun elmo-archive-tgz-cp-func (args &optional output)
   (elmo-archive-tgz-common-func args 'append t))
@@ -774,8 +756,7 @@ TYPE specifies the archiver's symbol."
 	(setq header-end (point))
       (setq header-end (point-max)))
     (narrow-to-region (point-min) header-end)
-    (elmo-msgdb-create-overview-from-buffer number)
-    ))
+    (elmo-msgdb-create-overview-from-buffer number)))
 
 (defsubst elmo-archive-msgdb-create-entity (method archive number type &optional prefix) ;; verrrry slow!!
   (let* ((msg (elmo-concat-path prefix (int-to-string number)))
@@ -817,7 +798,7 @@ TYPE specifies the archiver's symbol."
       (set-buffer tmp-buf)
       (setq num (length numlist))
       (setq i 0)
-      (message "creating msgdb...")
+      (message "Creating msgdb...")
       (while numlist
 	(erase-buffer)
 	(setq entity
@@ -850,10 +831,10 @@ TYPE specifies the archiver's symbol."
 		     gmark))))
 	(setq i (1+ i))
 	(setq percent (/ (* i 100) num))
-	(message "creating msgdb...%d%%" percent)
+	(message "Creating msgdb...%d%%" percent)
 	(setq numlist (cdr numlist)))
       (kill-buffer tmp-buf)
-      (message "creating msgdb...done.")
+      (message "Creating msgdb...done.")
       (list overview number-alist mark-alist)) ))
 
 ;;; info-zip agent
@@ -875,7 +856,7 @@ TYPE specifies the archiver's symbol."
     (set-buffer buf)
     (setq num (length numlist))
     (setq i 0)
-    (message "creating msgdb...")
+    (message "Creating msgdb...")
     (while numlist
       (setq n (min (1- elmo-archive-fetch-headers-volume)
 		   (1- (length numlist))))
@@ -910,7 +891,7 @@ TYPE specifies the archiver's symbol."
 	(error "unknown format!")))
       (setq i (+ n i))
       (setq percent (/ (* i 100) num))
-      (message "creating msgdb...%d%%" percent))
+      (message "Creating msgdb...%d%%" percent))
     (kill-buffer buf)
     (list overview number-alist mark-alist)) )
 
@@ -968,55 +949,39 @@ TYPE specifies the archiver's symbol."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Search functions
 
-(defsubst elmo-archive-field-string-match (spec number string field prefix)
+(defsubst elmo-archive-field-condition-match (spec number condition prefix)
   (save-excursion
     (let* ((type (nth 2 spec))
 	   (arc (elmo-archive-get-archive-name (nth 1 spec) type spec))
 	   (method (elmo-archive-get-method type 'cat))
-	   (args (list arc (elmo-concat-path prefix (int-to-string number))))
-	   (tmp-buffer (get-buffer-create " *ELMO ARCHIVE grep*"))
-	   ret-val fval)
-      (set-buffer tmp-buffer)
-      (erase-buffer)
-      (when (file-exists-p arc)
-	(as-binary-process
-	 (elmo-archive-call-method method args t))
-	(elmo-set-buffer-multibyte default-enable-multibyte-characters)
-	(decode-mime-charset-region (point-min)(point-max) elmo-mime-charset)
-	(goto-char (point-min))
-	(cond 
-	 ((string-match "body" field)
-	  (re-search-forward "^$" nil t)	   ; goto body
-	  (setq ret-val (search-forward string nil t)))
-	 (t
-	  (if (setq fval (std11-field-body field))
-	      (setq ret-val (string-match string fval)))))
-	(kill-buffer tmp-buffer)
-	ret-val))))
+	   (args (list arc (elmo-concat-path prefix (int-to-string number)))))
+      (elmo-set-work-buf
+       (when (file-exists-p arc)
+	 (as-binary-process
+	  (elmo-archive-call-method method args t))
+	 (elmo-set-buffer-multibyte default-enable-multibyte-characters)
+	 (decode-mime-charset-region (point-min)(point-max) elmo-mime-charset)
+	 (elmo-buffer-field-condition-match condition))))))
 
-
-(defun elmo-archive-search (spec key &optional from-msgs)
-  (when (string-match "\\([^ ]+\\) *\\(.*\\)$" key)
-    (let* (;;(args (elmo-string-to-list key))
-	   ;; XXX: I don't know whether `elmo-archive-list-folder'
-	   ;;      updates match-data.
-	   ;; (msgs (or from-msgs (elmo-archive-list-folder spec)))
-	   (search-key (elmo-match-string 1 key))
-	   (search-value (elmo-match-string 2 key))
-	   (msgs (or from-msgs (elmo-archive-list-folder spec)))
-	   (num (length msgs))
-	   (i 0)
-	   (case-fold-search nil)
-	   ret-val)
-      (while msgs
-	(if (elmo-archive-field-string-match spec (car msgs)
-					     search-value search-key
-					     (nth 3 spec))
-	    (setq ret-val (cons (car msgs) ret-val)))
-	(setq i (1+ i))
-	(message "searching...%d%%" (/ (* i 100) num))
-	(setq msgs (cdr msgs)))
-      (nreverse ret-val))))
+(defun elmo-archive-search (spec condition &optional from-msgs)
+  (let* (;;(args (elmo-string-to-list key))
+	 ;; XXX: I don't know whether `elmo-archive-list-folder'
+	 ;;      updates match-data.
+	 ;; (msgs (or from-msgs (elmo-archive-list-folder spec)))
+	 (msgs (or from-msgs (elmo-archive-list-folder spec)))
+	 (num (length msgs))
+	 (i 0)
+	 (case-fold-search nil)
+	 ret-val)
+    (while msgs
+      (if (elmo-archive-field-condition-match spec (car msgs)
+					      condition
+					      (nth 3 spec))
+	  (setq ret-val (cons (car msgs) ret-val)))
+      (setq i (1+ i))
+      (message "Searching...%d%%" (/ (* i 100) num))
+      (setq msgs (cdr msgs)))
+    (nreverse ret-val)))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1056,13 +1021,25 @@ TYPE specifies the archiver's symbol."
 	(setq slist (cdr slist)))))
 
 (defun elmo-archive-use-cache-p (spec number)
-  elmo-archive-use-cache
-  )
+  elmo-archive-use-cache)
 
 (defun elmo-archive-local-file-p (spec number)
-  nil
-  )
+  nil)
 
+(defun elmo-archive-get-msg-filename (spec number &optional loc-alist)
+  (let ((tmp-dir (file-name-as-directory (elmo-msgdb-expand-path nil spec)))
+	(prefix (nth 3 spec)))
+    (expand-file-name
+     (elmo-concat-path prefix (int-to-string number))
+     tmp-dir)))
+
+(defalias 'elmo-archive-sync-number-alist 
+  'elmo-generic-sync-number-alist)
+(defalias 'elmo-archive-list-folder-unread 
+  'elmo-generic-list-folder-unread)
+(defalias 'elmo-archive-list-folder-important
+  'elmo-generic-list-folder-important)
+(defalias 'elmo-archive-commit 'elmo-generic-commit)
 
 ;;; End
 (run-hooks 'elmo-archive-load-hook)
