@@ -1,22 +1,34 @@
-;;;
 ;;; elmo-dop.el -- Modules for Disconnected Operations on ELMO.
-;;; Copyright (C) 1998 Yuuichi Teranishi <teranisi@gohome.org>
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
+
+;; Copyright 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, net news
+;; Time-stamp: <00/03/14 19:39:23 teranisi>
+
+;; This file is part of ELMO (Elisp Library for Message Orchestration).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
+
+;;; Code:
+;; 
 
 (require 'elmo-vars)
 (require 'elmo-msgdb)
@@ -31,18 +43,28 @@
 Automatically loaded/saved.")
 
 (defun elmo-dop-queue-append (folder function argument)
-  (elmo-dop-queue-load)
-  (setq elmo-dop-queue 
-	(append elmo-dop-queue
-		(list (list (format "%s" folder) function argument))))
-  (elmo-dop-queue-save))
+  (let ((operation (list (format "%s" folder) function argument)))
+    (elmo-dop-queue-load)
+    (unless (member operation elmo-dop-queue) ;; don't append same operation
+      (setq elmo-dop-queue 
+	    (append elmo-dop-queue
+		    (list operation)))
+      (elmo-dop-queue-save))))
 
 (defun elmo-dop-queue-flush ()
   (elmo-dop-queue-load) ; load cache.
-  (if elmo-dop-queue
-      (if (y-or-n-p 
-	   (format "%d pending operation(s) exists. Perform now?" 
-		   (length elmo-dop-queue)))
+  (elmo-dop-queue-merge)
+  (let ((queue elmo-dop-queue)
+	(count 0)
+	len)
+    (while queue
+      (if (elmo-folder-plugged-p (caar queue))
+	  (setq count (1+ count)))
+      (setq queue (cdr queue)))
+    (when (> count 0)
+      (if (elmo-y-or-n-p 
+	   (format "%d pending operation(s) exists. Perform now?" count)
+	   (not elmo-dop-flush-confirm) t)
 	  (progn
 	    (message "")
 	    (sit-for 0)
@@ -50,42 +72,90 @@ Automatically loaded/saved.")
 		  (performed 0)
 		  (i 0)
 		  (num (length elmo-dop-queue))
-		  failure)
+		  folder func failure)
 	      (while queue
 		;; now perform pending processes.
 		(setq failure nil)
-		(condition-case ()
-		    (elmo-call-func
-		     (nth 0 (car queue))
-		     (nth 1 (car queue))
-		     (nth 2 (car queue)) ;argunemt
-		     )		
-		  (quit  (setq failure t))
-		  (error (setq failure t)))
 		(setq i (+ 1 i))
-		(message "%d/%d operation(s) has performed." i num)
+		(message "Flushing queue....%d/%d." i num)
+		(condition-case err
+		    (if (not (elmo-folder-plugged-p (nth 0 (car queue))))
+			(setq failure t)
+		      (setq folder (nth 0 (car queue))
+			    func (nth 1 (car queue)))
+		      (cond
+		       ((string= func "prefetch-msgs")
+			(elmo-prefetch-msgs
+			 folder
+			 (nth 2 (car queue)))) ;argunemt
+		       ((string= func "append-operations")
+			(elmo-dop-flush-pending-append-operations
+			 folder nil t))
+		       (t
+			(elmo-call-func
+			 folder
+			 func
+			 (nth 2 (car queue)) ;argunemt
+			 ))))
+		  (quit  (setq failure t))
+		  (error (setq failure err)))
 		(if failure 
 		    ;; create-folder was failed.
-		    (when (and (string= (nth 1 (car queue)) 
-					"create-folder-maybe")
-			       (y-or-n-p 
+		    (when (and (string= func "create-folder-maybe")
+			       (elmo-y-or-n-p 
 				(format 
 				 "Create folder %s failed. Abort creating?"
-				 (nth 0 (car queue)) elmo-lost+found-folder)))
-		      (elmo-dop-save-pending-messages
-		       (nth 0 (car queue)))
+				 folder)
+				(not elmo-dop-flush-confirm) t))
+		      (elmo-dop-save-pending-messages folder)
 		      (setq elmo-dop-queue (delq (car queue) elmo-dop-queue)))
 		  (setq elmo-dop-queue (delq (car queue) elmo-dop-queue))
 		  (setq performed (+ 1 performed)))
-		(setq queue (cdr queue))
-		)
-	      (message "%d operation(s) are performed successfully." performed)
+		(setq queue (cdr queue)))
+	      (message "%d/%d operation(s) are performed successfully."
+		       performed num)
+	      (sit-for 1) ; 
 	      (elmo-dop-queue-save)))
-	(if (y-or-n-p "Clear all pending operations?")
-	    (progn
+	(if (elmo-y-or-n-p "Clear all pending operations?"
+			   (not elmo-dop-flush-confirm) t)
+	    (let ((queue elmo-dop-queue))
+	      (while queue
+		(if (string= (nth 1 (car queue)) "append-operations")
+		    (elmo-dop-append-list-save (nth 0 (car queue)) nil))
+		(setq queue (cdr queue)))
 	      (setq elmo-dop-queue nil)
 	      (message "All pending operations are cleared.")
-	      (elmo-dop-queue-save))))))
+	      (elmo-dop-queue-save))
+	  (message "")))
+      count)))
+
+(defconst elmo-dop-merge-funcs
+  '("delete-msgids"
+    "prefetch-msgs"
+    "unmark-important"
+    "mark-as-important"
+    "mark-as-read"
+    "mark-as-unread"))
+
+(defun elmo-dop-queue-merge ()
+  (let ((queue elmo-dop-queue)
+        new-queue match-queue que)
+    (while (setq que (car queue))
+      (if (and 
+	   (member (cadr que) elmo-dop-merge-funcs)
+	   (setq match-queue
+		 (car (delete nil
+			      (mapcar '(lambda (new-queue)
+					 (if (and 
+					      (string= (car que) (car new-queue))
+					      (string= (cadr que) (cadr new-queue)))
+					     new-queue))
+				      new-queue)))))
+	  (setcar (cddr match-queue)
+		  (append (nth 2 match-queue) (nth 2 que)))
+	(setq new-queue (append new-queue (list que))))
+      (setq queue (cdr queue)))
+    (setq elmo-dop-queue new-queue)))
 
 (defun elmo-dop-queue-load ()
   (save-excursion
@@ -135,19 +205,25 @@ Automatically loaded/saved.")
 		     elmo-msgdb-dir)
    lock-list))
 
-(defun elmo-dop-append-list-load (folder)
+(defun elmo-dop-append-list-load (folder &optional resume)
   (elmo-object-load 
-   (expand-file-name elmo-msgdb-append-list-filename
+   (expand-file-name (if resume
+			 elmo-msgdb-resume-list-filename
+		       elmo-msgdb-append-list-filename)
 		     (elmo-msgdb-expand-path folder))))
 
-(defun elmo-dop-append-list-save (folder append-list)
+(defun elmo-dop-append-list-save (folder append-list &optional resume)
   (if append-list
       (elmo-object-save
-       (expand-file-name elmo-msgdb-append-list-filename
+       (expand-file-name (if resume
+			     elmo-msgdb-resume-list-filename
+			   elmo-msgdb-append-list-filename)
 			 (elmo-msgdb-expand-path folder))
        append-list)
     (condition-case ()
-	(delete-file (expand-file-name elmo-msgdb-append-list-filename
+	(delete-file (expand-file-name (if resume
+					   elmo-msgdb-resume-list-filename
+					 elmo-msgdb-append-list-filename)
 				       (elmo-msgdb-expand-path folder)))
       (error))))
 
@@ -182,18 +258,21 @@ Automatically loaded/saved.")
 		       "delete-msgids" ;; for secure removal.
 		       (cdr appended-deleting)))
 		  (elmo-dop-append-list-save folder (car appended-deleting)))
-	      (error "Unplugged."))
+	      (error "Unplugged"))
 	  ;; not imap4 folder...delete now!
 	  (elmo-call-func (car (car folder-numbers)) "delete-msgs"
 			  (cdr (car folder-numbers))))
 	(setq folder-numbers (cdr folder-numbers))))
     t))
 
+(defun elmo-dop-prefetch-msgs (folder msgs)
+  (save-match-data
+    (elmo-dop-queue-append folder "prefetch-msgs" msgs)))
+
 (defun elmo-dop-list-folder (folder)
-  (if (or (eq (elmo-folder-get-type folder) 'imap4)
-	  (eq (elmo-folder-get-type folder) 'nntp)
-	  (eq (elmo-folder-get-type folder) 'filter)
-	  (elmo-multi-p folder))
+  (if (or (memq (elmo-folder-get-type folder)
+		'(imap4 nntp pop3 filter pipe))
+	  (and (elmo-multi-p folder) (not (elmo-folder-local-p folder))))
       (if elmo-enable-disconnected-operation
 	  (let* ((number-alist (elmo-msgdb-number-load
 				(elmo-msgdb-expand-path folder)))
@@ -207,8 +286,7 @@ Automatically loaded/saved.")
 	      (if (rassoc (car append-list) number-alist)
 		  (setq alreadies (append alreadies 
 					  (list (car append-list)))))
-	      (setq append-list (cdr append-list))
-	      )
+	      (setq append-list (cdr append-list)))
 	    (setq append-num (- append-num (length alreadies)))
 	    (setq max-num 
 		  (or (nth (max (- (length number-list) 1) 0) 
@@ -219,7 +297,7 @@ Automatically loaded/saved.")
 			    (list (+ max-num i 1))))
 	      (setq i (+ 1 i)))
 	    number-list)
-	(error "Unplugged."))
+	(error "Unplugged"))
     ;; not imap4 folder...list folder
     (elmo-call-func folder "list-folder")))
 
@@ -260,14 +338,13 @@ Automatically loaded/saved.")
 	      (if (rassoc (car append-list) number-alist)
 		  (setq alreadies (append alreadies 
 					  (list (car append-list)))))
-	      (setq append-list (cdr append-list))
-	      )
+	      (setq append-list (cdr append-list)))
 	    (setq max-num 
 		  (or (nth (max (- (length number-list) 1) 0) number-list)
 		      0))
 	    (cons (- (+ max-num append-num) (length alreadies))
 		  (- (+ (length number-list) append-num) (length alreadies))))
-	(error "Unplugged."))
+	(error "Unplugged"))
     ;; not imap4 folder.
     (elmo-call-func folder "max-of-folder")))
 
@@ -282,14 +359,11 @@ Automatically loaded/saved.")
 	(elmo-append-msg elmo-lost+found-folder file-string)
 	(elmo-dop-unlock-message (car append-list)))
       (setq append-list (cdr append-list))
-      (elmo-dop-append-list-save folder nil)
-      )
-    )
+      (elmo-dop-append-list-save folder nil)))
   (message (format "Saving queued message in %s...done." 
-		   elmo-lost+found-folder))
-  )
+		   elmo-lost+found-folder)))
 
-(defun elmo-dop-flush-pending-append-operations (folder &optional appends)
+(defun elmo-dop-flush-pending-append-operations (folder &optional appends resume)
   (message "Appending queued messages...")
   (let* ((append-list (or appends 
 			  (elmo-dop-append-list-load folder)))
@@ -297,6 +371,19 @@ Automatically loaded/saved.")
 	 (i 0)
 	 (num (length append-list))
 	 failure file-string)
+    (when resume
+      ;; Resume msgdb changed by elmo-dop-msgdb-create.
+      (let* ((resumed-list (elmo-dop-append-list-load folder t))
+	     (number-alist (elmo-msgdb-number-load 
+			    (elmo-msgdb-expand-path folder)))
+	     (appendings append-list)
+	     pair dels)
+	(while appendings
+	  (if (setq pair (rassoc (car appendings) number-alist))
+	      (setq resumed-list (append resumed-list
+					 (list (car appendings)))))
+	  (setq appendings (cdr appendings)))
+	(elmo-dop-append-list-save folder resumed-list t)))
     (while appendings
       (setq failure nil)
       (setq file-string (elmo-get-file-string  ; message string
@@ -316,8 +403,14 @@ Automatically loaded/saved.")
       (setq appendings (cdr appendings)))
     ;; All pending append operation is flushed.
     (elmo-dop-append-list-save folder nil)
-    (elmo-imap4-commit folder)
-    )
+    (elmo-commit folder)
+    (unless resume
+      ;; delete '(folder "append-operations") in elmo-dop-queue.
+      (let (elmo-dop-queue)
+	(elmo-dop-queue-load)
+	(setq elmo-dop-queue (delete (list folder "append-operations" nil)
+				     elmo-dop-queue))
+	(elmo-dop-queue-save))))
   (message "Appending queued messages...done."))
 
 (defun elmo-dop-folder-exists-p (folder)
@@ -330,25 +423,37 @@ Automatically loaded/saved.")
   (if (eq (elmo-folder-get-type folder) 'imap4)
       (if elmo-enable-disconnected-operation
 	  (elmo-dop-queue-append folder "create-folder-maybe" nil)
-	(error "Unplugged."))
+	(error "Unplugged"))
     (elmo-call-func folder "create-folder")))
 
+(defun elmo-dop-delete-folder (folder)
+  (error "Unplugged"))
+
+(defun elmo-dop-rename-folder (old-folder new-folder)
+  (error "Unplugged"))
+
 (defun elmo-dop-append-msg (folder string message-id &optional msg)
   (if elmo-enable-disconnected-operation
       (if message-id
-	  (let ((append-list (elmo-dop-append-list-load folder))
-		(number-alist (elmo-msgdb-number-load 
-			       (elmo-msgdb-expand-path folder))))
-	    (when (and ; not in current folder.
-		   (not (rassoc message-id number-alist))
-		   (not (member message-id append-list)))
-	      (setq append-list
-		    (append append-list (list message-id)))
-	      (elmo-dop-lock-message message-id)
-	      (elmo-dop-append-list-save folder append-list))
-	    t)
+	  (progn
+	    (unless (elmo-cache-exists-p message-id)
+	      (elmo-set-work-buf
+	       (insert string)
+	       (elmo-cache-save message-id nil folder msg (current-buffer))))
+	    (let ((append-list (elmo-dop-append-list-load folder))
+		  (number-alist (elmo-msgdb-number-load 
+				 (elmo-msgdb-expand-path folder))))
+	      (when (and ; not in current folder.
+		     (not (rassoc message-id number-alist))
+		     (not (member message-id append-list)))
+		(setq append-list
+		      (append append-list (list message-id)))
+		(elmo-dop-lock-message message-id)
+		(elmo-dop-append-list-save folder append-list)
+		(elmo-dop-queue-append folder "append-operations" nil))
+	      t))
 	nil)
-    (error "Unplugged.")))
+    (error "Unplugged")))
 
 (defalias 'elmo-dop-msgdb-create 'elmo-dop-msgdb-create-as-numlist)
 
@@ -431,12 +536,14 @@ Automatically loaded/saved.")
 			     mark-alist (car numlist) gmark)))))
 	      (setq i (1+ i))
 	      (setq percent (/ (* i 100) num))
-	      (message "creating msgdb...%d%%" percent)
+	      (message "Creating msgdb...%d%%" percent)
 	      (setq numlist (cdr numlist)))
 	    (list overview number-alist mark-alist))
-	(error "Unplugged."))
+	(error "Unplugged"))
     ;; not imap4 folder...
     (elmo-call-func folder "msgdb-create" numlist new-mark already-mark
 		    seen-mark important-mark seen-list)))
 
 (provide 'elmo-dop)
+
+;;; elmo-dop.el ends here
