@@ -1,22 +1,34 @@
-;;;
 ;;; elmo-localdir.el -- Localdir Interface for ELMO.
-;;; Copyright (C) 1998 Yuuichi Teranishi <teranisi@gohome.org>
-;;;
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
+
+;; Copyright 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, net news
+;; Time-stamp: <00/03/22 00:03:39 teranisi>
+
+;; This file is part of ELMO (Elisp Library for Message Orchestration).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
+
+;;; Code:
+;; 
 
 (require 'emu)
 (require 'std11)
@@ -25,12 +37,14 @@
   (require 'elmo-cache))
 (require 'elmo-msgdb)
 
-
 (defsubst elmo-localdir-get-folder-directory (spec)
-  (if (string-match "^/.*\\|^[A-Z]:" (nth 1 spec))
+  (if (file-name-absolute-p (nth 1 spec))
       (nth 1 spec) ; already full path.
     (expand-file-name (nth 1 spec)
-		      elmo-localdir-folder-path)))
+		      (cond ((eq (car spec) 'localnews)
+			     elmo-localnews-folder-path)
+			    (t
+			     elmo-localdir-folder-path)))))
 
 (defun elmo-localdir-msgdb-expand-path (spec)
   (let ((fld-name (nth 1 spec)))
@@ -38,6 +52,9 @@
 		      (expand-file-name "localdir"
 					elmo-msgdb-dir))))
 
+(defun elmo-localdir-number-to-filename (spec dir number &optional loc-alist)
+  (expand-file-name (int-to-string number) dir))
+
 (if (boundp 'nemacs-version)
     (defsubst elmo-localdir-insert-header (file)
       "Insert the header of the article (Does not work on nemacs)."
@@ -60,17 +77,21 @@
 		    (prog1 (not (search-forward "\n\n" nil t))
 		      (goto-char (point-max)))))))))
 
+
 (defsubst elmo-localdir-msgdb-create-overview-entity-from-file (number file)
   (save-excursion
     (let ((tmp-buffer (get-buffer-create " *ELMO LocalDir Temp*"))
 	  insert-file-contents-pre-hook   ; To avoid autoconv-xmas...
 	  insert-file-contents-post-hook header-end
-	  ret-val size)
+	  (attrib (file-attributes file))
+	  ret-val size mtime)
       (set-buffer tmp-buffer)
       (erase-buffer)
       (if (not (file-exists-p file))
 	  ()
-	(setq size (nth 7 (file-attributes file)))
+	(setq size (nth 7 attrib))
+	(setq mtime (timezone-make-date-arpa-standard
+		     (current-time-string (nth 5 attrib)) (current-time-zone)))
 	;; insert header from file.
 	(catch 'done
 	  (condition-case nil
@@ -82,7 +103,7 @@
 		    (point)
 		  (point-max)))
 	  (narrow-to-region (point-min) header-end)
-	  (setq ret-val (elmo-msgdb-create-overview-from-buffer number size))
+	  (setq ret-val (elmo-msgdb-create-overview-from-buffer number size mtime))
 	  (kill-buffer tmp-buffer))
 	ret-val
 	))))
@@ -100,7 +121,7 @@
 	  i percent len num seen gmark)
       (setq len (length numlist))
       (setq i 0)
-      (message "creating msgdb...")
+      (message "Creating msgdb...")
       (while numlist
 	(setq entity
 	      (elmo-localdir-msgdb-create-entity 
@@ -133,15 +154,18 @@
 		     gmark))))
 	(setq i (1+ i))
 	(setq percent (/ (* i 100) len))
-	(message "creating msgdb...%d%%" percent)
+	(message "Creating msgdb...%d%%" percent)
 	(setq numlist (cdr numlist)))
-      (message "creating msgdb...done.")
+      (message "Creating msgdb...done.")
       (list overview number-alist mark-alist))))
 
 (defalias 'elmo-localdir-msgdb-create 'elmo-localdir-msgdb-create-as-numlist)
 
+(defvar elmo-localdir-list-folders-spec-string "+")
+(defvar elmo-localdir-list-folders-filter-regexp "^\\(\\.\\.?\\|[0-9]+\\)$")
+
 (defun elmo-localdir-list-folders (spec &optional hierarchy)
-  (let ((folder (concat "+" (nth 1 spec))))
+  (let ((folder (concat elmo-localdir-list-folders-spec-string (nth 1 spec))))
     (elmo-localdir-list-folders-subr folder hierarchy)))
 
 (defun elmo-localdir-list-folders-subr (folder &optional hierarchy)
@@ -153,7 +177,7 @@
 	  (setq curdir
 		(expand-file-name (nth 1 (elmo-folder-get-spec folder))
 				  elmo-localdir-folder-path))
-	  (if (string-match "^[+=$]$" folder) ;; localdir, archive, localnews
+	  (if (string-match "^[+=$.]$" folder) ; localdir, archive, localnews
 	      (setq subprefix folder)
 	    (setq subprefix (concat folder elmo-path-sep))
 	    ;; include parent
@@ -165,7 +189,9 @@
 	    (setq dirent (cdr dirent))
 	    (setq abspath (expand-file-name relpath curdir))
 	    (and
-	     (not (string-match "^\\(\\.\\.?\\|[0-9]+\\)$" relpath))
+	     (not (string-match 
+		   elmo-localdir-list-folders-filter-regexp
+		   relpath))
 	     (eq (nth 0 (setq attr (file-attributes abspath))) t)
 	     (if (eq hierarchy 'check)
 		 (throw 'done (nconc folders t))
@@ -191,13 +217,13 @@
 
 (defsubst elmo-localdir-list-folder-subr (spec &optional nonsort)
   (let* ((dir (elmo-localdir-get-folder-directory spec))
-	 (flist (mapcar 'string-to-int (directory-files dir nil "^[0-9]+$" t))))
+	 (flist (mapcar 'string-to-int
+			(directory-files dir nil "^[0-9]+$" t))))
     (if nonsort
 	(cons (or (elmo-max-of-list flist) 0) (length flist))
-      (sort flist '<))
-    ))
+      (sort flist '<))))
 
-(defun elmo-localdir-append-msg (spec string &optional msg)
+(defun elmo-localdir-append-msg (spec string &optional msg no-see)
   (let ((dir (elmo-localdir-get-folder-directory spec))
 	(tmp-buffer (get-buffer-create " *ELMO Temp buffer*"))
 	(next-num (or msg
@@ -219,7 +245,6 @@
 	    nil
 	    )
 	(kill-buffer tmp-buffer)))))
-	
 
 (defun elmo-localdir-delete-msg (spec number)
   (let (file
@@ -242,7 +267,7 @@
       (erase-buffer)
       (when (file-exists-p file)
 	(as-binary-input-file (insert-file-contents file))
-	(elmo-delete-cr-get-content-type outbuf)))))
+	(elmo-delete-cr-get-content-type)))))
 
 (defun elmo-localdir-delete-msgs (spec msgs)
   (mapcar '(lambda (msg) (elmo-localdir-delete-msg spec msg))
@@ -285,78 +310,83 @@
 (defun elmo-localdir-folder-exists-p (spec)
   (file-directory-p (elmo-localdir-get-folder-directory spec)))
 
+(defun elmo-localdir-folder-creatable-p (spec)
+  t)
+
 (defun elmo-localdir-create-folder (spec)
   (save-excursion
     (let ((dir (elmo-localdir-get-folder-directory spec)))
       (if (file-directory-p dir)
           ()
 	(if (file-exists-p dir)
-	    (error "Create folder failed.")
+	    (error "Create folder failed")
 	  (elmo-make-directory dir))
 	t
 	))))
 
-(defsubst elmo-localdir-field-string-match (spec number string field)
-  (elmo-file-field-string-match 
+(defun elmo-localdir-delete-folder (spec)
+  (let* ((dir (elmo-localdir-get-folder-directory spec)))
+    (if (not (file-directory-p dir))
+	(error "no such directory: %s" dir)
+      (elmo-delete-directory dir t)
+      t)))
+
+(defun elmo-localdir-rename-folder (old-spec new-spec)
+  (let* ((old (elmo-localdir-get-folder-directory old-spec))
+	 (new (elmo-localdir-get-folder-directory new-spec))
+	 (new-dir (directory-file-name (file-name-directory new))))
+    (if (not (file-directory-p old))
+	(error "no such directory: %s" old)
+      (if (file-exists-p new)
+	  (error "already exists directory: %s" new)
+	(if (not (file-exists-p new-dir))
+	    (elmo-make-directory new-dir))
+	(rename-file old new)
+	t))))
+
+(defsubst elmo-localdir-field-condition-match (spec number condition)
+  (elmo-file-field-condition-match 
    (expand-file-name (int-to-string number)
 		     (elmo-localdir-get-folder-directory spec))
-   string
-   field))
+   condition))
 
-(defun elmo-localdir-search (spec key &optional from-msgs)
-  (when (string-match "\\([^ ]+\\) *\\(.*\\)$" key)
-    (let* ((search-key (elmo-match-string 1 key))
-	   (search-value (elmo-match-string 2 key))
-	   (msgs (or from-msgs (elmo-localdir-list-folder spec)))
-	   (num (length msgs))
-	   (i 0)
-	   case-fold-search 
-	   ret-val)
-      (while msgs
-	(if (elmo-localdir-field-string-match spec (car msgs)
-					      search-value search-key)
-	    (setq ret-val (cons (car msgs) ret-val)))
-	(setq i (1+ i))
-	(message "searching...%d%%" (/ (* i 100) num))
-	(setq msgs (cdr msgs)))
-      (nreverse ret-val))))
+(defun elmo-localdir-search (spec condition &optional from-msgs)
+  (let* ((msgs (or from-msgs (elmo-localdir-list-folder spec)))
+	 (num (length msgs))
+	 (i 0) case-fold-search ret-val)
+    (while msgs
+      (if (elmo-localdir-field-condition-match spec (car msgs)
+					       condition)
+	  (setq ret-val (cons (car msgs) ret-val)))
+      (setq i (1+ i))
+      (message "Searching...%d%%" (/ (* i 100) num))
+      (setq msgs (cdr msgs)))
+    (nreverse ret-val)))
 
-(defsubst elmo-localdir-copy-msgs-internal (src-spec msgs dst-spec expunge-it
-						     &optional same-number)
-  (let ((mlist msgs)
-	(dst-dir 
+;;; (localdir, maildir, localnews) -> localdir
+(defun elmo-localdir-copy-msgs (dst-spec msgs src-spec
+					 &optional loc-alist same-number)
+  (let ((dst-dir
 	 (elmo-localdir-get-folder-directory dst-spec))
 	(next-num (1+ (car (elmo-localdir-list-folder-subr dst-spec t)))))
-    (while mlist
+    (while msgs
       (elmo-copy-file
        ;; src file
-       (expand-file-name (int-to-string (car mlist))
-			 (elmo-localdir-get-folder-directory 
-			  src-spec))
+       (elmo-call-func src-spec "get-msg-filename" (car msgs) loc-alist)
        ;; dst file
-       (expand-file-name
-	(int-to-string
-	 (if same-number
-	     (car mlist)
-	   next-num))
-	dst-dir))
-      (if (setq mlist (cdr mlist))
+       (expand-file-name (int-to-string
+			  (if same-number (car msgs) next-num))
+			 dst-dir))
+      (if (and (setq msgs (cdr msgs))
+	       (not same-number))
 	  (setq next-num
-		(if (elmo-localdir-locked-p)
+		(if (and (eq (car dst-spec) 'localdir)
+			 (elmo-localdir-locked-p))
 		    ;; MDA is running.
 		    (1+ (car (elmo-localdir-list-folder-subr dst-spec t)))
 		  (1+ next-num)))))
     t))
 
-(defun elmo-localdir-copy-msgs (src-spec msgs dst-spec expunge-it
-					 &optional same-number)
-  (if (eq 'archive (nth 0 dst-spec))
-      (progn
-	(require 'elmo-archive)
-	;; cannot silence bytecompiler..??
-	(elmo-archive-copy-msgs src-spec msgs dst-spec expunge-it same-number))
-    (elmo-localdir-copy-msgs-internal src-spec msgs dst-spec expunge-it same-number)))
-
 (defun elmo-localdir-pack-number (spec msgdb arg)
   (let ((dir (elmo-localdir-get-folder-directory spec))
 	(onum-alist (elmo-msgdb-get-number-alist msgdb))
@@ -370,7 +400,7 @@
 	    (mapcar 'car onum-alist)))
     (setq total (length flist))
     (while flist
-      (message "packing...%d%%" (/ (* i 100) total))
+      (message "Packing...%d%%" (/ (* i 100) total))
       (setq onum (car flist))
       (when (not (eq onum i)) ;; why \=() is wrong..
         (elmo-bind-directory
@@ -391,21 +421,19 @@
 	       i mark)))
       (setq i (1+ i))
       (setq flist (cdr flist)))
-    (message "packing...done.")
+    (message "Packing...done.")
     (list (elmo-msgdb-get-overview msgdb)
 	  onum-alist
 	  new-mark-alist
 	  (elmo-msgdb-get-location msgdb))))
 
 (defun elmo-localdir-use-cache-p (spec number)
-  nil
-  )
+  nil)
 
 (defun elmo-localdir-local-file-p (spec number)
-  t
-  )
+  t)
 
-(defun elmo-localdir-get-msg-filename (spec number)
+(defun elmo-localdir-get-msg-filename (spec number &optional loc-alist)
   (expand-file-name 
    (int-to-string number)
    (elmo-localdir-get-folder-directory spec)))
@@ -419,5 +447,14 @@
 		(throw 'found t))
 	    (setq lock (cdr lock)))))))
 
+(defalias 'elmo-localdir-sync-number-alist 
+  'elmo-generic-sync-number-alist)
+(defalias 'elmo-localdir-list-folder-unread 
+  'elmo-generic-list-folder-unread)
+(defalias 'elmo-localdir-list-folder-important
+  'elmo-generic-list-folder-important)
+(defalias 'elmo-localdir-commit 'elmo-generic-commit)
+
 (provide 'elmo-localdir)
 
+;;; elmo-localdir.el ends here
