@@ -6,7 +6,10 @@
   (require 'elmo-vars)
   (require 'elmo-util)
   (require 'bbdb)
-  (require 'wl)
+  (require 'wl-summary)
+  (require 'wl-message)
+  (require 'wl-draft)
+  (require 'wl-address)
   (defvar bbdb-pop-up-elided-display nil)
   (or (fboundp 'bbdb-extract-field-value-internal)
       (defun bbdb-extract-field-value-internal (field))))
@@ -21,20 +24,22 @@
   (add-hook 'wl-exit-hook 'bbdb-wl-exit)
   (add-hook 'wl-summary-toggle-disp-off-hook 'bbdb-wl-hide-bbdb-buffer)
   (add-hook 'wl-summary-toggle-disp-folder-on-hook 'bbdb-wl-hide-bbdb-buffer)
+  (add-hook 'wl-summary-toggle-disp-folder-off-hook 'bbdb-wl-hide-bbdb-buffer)
+  (add-hook 'wl-summary-toggle-disp-folder-message-resumed-hook
+	    'bbdb-wl-show-bbdb-buffer)
   (add-hook 'wl-summary-mode-hook
 	    (function
 	     (lambda ()
 	       (define-key (current-local-map) ":" 'bbdb-wl-show-sender)
-	       (define-key (current-local-map) ";" 'bbdb-wl-edit-notes)
-	       (bbdb-flush-all-caches)
-	       )))
+	       (define-key (current-local-map) ";" 'bbdb-wl-edit-notes))))
+  (add-hook 'wl-summary-exit-hook 'bbdb-flush-all-caches)
   (add-hook 'wl-summary-exec-hook 'bbdb-flush-all-caches)
   (add-hook 'wl-mail-setup-hook
 	    (function
 	     (lambda ()
 ;	       (local-set-key "\M-\t" 'bbdb-complete-name)
 	       (define-key (current-local-map) "\M-\t" 'bbdb-complete-name)
-	       ))) )
+	       ))))
 
 (defun bbdb-wl-exit ()
   (let (bbdb-buf)
@@ -44,10 +49,8 @@
 
 (defun bbdb-wl-get-update-record ()
   (set-buffer (wl-message-get-original-buffer))
-  (elmo-set-buffer-multibyte default-enable-multibyte-characters)
   (bbdb-wl-update-record)
-  (run-hooks 'bbdb-wl-get-update-record-hook)
-  )
+  (run-hooks 'bbdb-wl-get-update-record-hook))
 
 (defun bbdb-wl-hide-bbdb-buffer ()
   (let (bbdb-buf bbdb-win)
@@ -55,6 +58,31 @@
 	(if (setq bbdb-win (get-buffer-window bbdb-buf))
 	    (delete-window bbdb-win)))))
 
+(defun bbdb-wl-show-bbdb-buffer ()
+  (save-selected-window
+    (if (get-buffer-window bbdb-buffer-name)
+	nil  
+      (let ((mes-win (get-buffer-window 
+		      (save-excursion
+			(if (buffer-live-p  wl-current-summary-buffer)
+			    (set-buffer wl-current-summary-buffer))
+			wl-message-buf-name)))
+	    (cur-win (selected-window))
+	    (b (current-buffer)))
+	(and mes-win (select-window mes-win))
+	(let ((size (min
+		     (- (window-height mes-win)
+			window-min-height 1)
+		     (- (window-height mes-win)
+			(max window-min-height
+			     (1+ bbdb-pop-up-target-lines))))))    
+	  (split-window mes-win (if (> size 0) size window-min-height)))
+	;; goto the bottom of the two...
+	(select-window (next-window))
+	;; make it display *BBDB*...
+	(let ((pop-up-windows nil))
+	  (switch-to-buffer (get-buffer-create bbdb-buffer-name)))))))
+
 (defun bbdb-wl-from-func (string)
   "A candidate for wl-summary-from-func..."
   (let ((hit (bbdb-search-simple nil (wl-address-header-extract-address
@@ -83,29 +111,33 @@ the user confirms the creation."
     (if bbdb-use-pop-up
 	(bbdb-wl-pop-up-bbdb-buffer offer-to-create)
       (let ((key 
-	     (save-excursion (set-buffer 
-			      (get-buffer wl-message-buf-name))
-			     wl-message-buffer-cur-number)))
+	     (save-excursion
+	       (set-buffer 
+		(save-excursion
+		  (if (buffer-live-p wl-current-summary-buffer)
+		      (set-buffer wl-current-summary-buffer))
+		  wl-message-buf-name))
+	       (intern (format
+			"%s-%d"
+			wl-current-summary-buffer
+			wl-message-buffer-cur-number)))))
 	(or (bbdb-message-cache-lookup key nil)
 	    (and key
-		 (let* ((from 
-			 (elmo-eword-decode-string
-			  (decode-mime-charset-string
-			   (or (std11-field-body "From")
-			       "")
-			   wl-mime-charset)))
-			(addr
-			 (and from
-			      (car (cdr
-				    (mail-extract-address-components from))))))
+		 (let* ((from (or (std11-field-body "From") ""))
+			(addr (and from
+				   (nth 1 (std11-extract-address-components
+					   from)))))
 		   (if (or (null from)
 			   (null addr)
 			   (string-match (bbdb-user-mail-names) addr))
-		       (setq from (or 
-				   (elmo-eword-decode-string
-				    (decode-mime-charset-string
-				     (or (std11-field-body "To") from "")
-				     wl-mime-charset)))))
+		       (setq from (or (std11-field-body "To") from)))
+		   (with-temp-buffer ; to keep raw buffer unibyte.
+		     (elmo-set-buffer-multibyte
+		      default-enable-multibyte-characters)
+		     (setq from (eword-decode-string
+				 (decode-mime-charset-string
+				  from
+				  wl-mime-charset))))
 		   (if from
 		       (bbdb-encache-message
 			key
@@ -146,21 +178,29 @@ This buffer will be in bbdb-mode, with associated keybindings."
   (let ((record (bbdb-wl-update-record t))
 	bbdb-win)
     (if record
-	(bbdb-display-records (list record))
+	(progn
+	  (bbdb-wl-pop-up-bbdb-buffer)
+	  (bbdb-display-records (list record)))
       (error "unperson"))
     (setq bbdb-win (get-buffer-window (get-buffer bbdb-buffer-name)))
-    (select-window bbdb-win)))
+    (and bbdb-win
+	 (select-window bbdb-win))))
+
 
 (defun bbdb-wl-pop-up-bbdb-buffer (&optional offer-to-create)
   "Make the *BBDB* buffer be displayed along with the WL window(s),
 displaying the record corresponding to the sender of the current message."
   (if (get-buffer-window bbdb-buffer-name)
       nil  
-    (let ((mes-win (get-buffer-window wl-message-buf-name))
+    (let ((mes-win (get-buffer-window 
+		    (save-excursion
+		      (if (buffer-live-p  wl-current-summary-buffer)
+			  (set-buffer wl-current-summary-buffer))
+		      wl-message-buf-name)))
 	  (cur-win (selected-window))
-	  (b (current-buffer))
-	  )
-      (select-window mes-win)
+	  (b (current-buffer)))
+      (and mes-win
+	   (select-window mes-win))
       (let ((size (min
 		   (- (window-height mes-win)
 		      window-min-height 1)
@@ -207,7 +247,7 @@ displaying the record corresponding to the sender of the current message."
       (defun bbdb-extract-field-value (field)
 	(let ((value (bbdb-extract-field-value-internal field)))
 	  (and value
-	       (elmo-eword-decode-string value))))
+	       (eword-decode-string value))))
       ))
 
 
