@@ -1,32 +1,39 @@
-;;;
-;;; Wanderlust -- Yet Another Message Interface on Emacsen.
-;;;
-;;; Copyright (C) 1998 Masahiro MURATA <muse@ba2.so-net.ne.jp>
-;;;                    Yuuichi Teranishi <teranisi@gohome.org>
-;;;
-;;; Time-stamp: <99/04/26 15:51:25 teranisi>
-;;;
-;;; Author: Masahiro MURATA <muse@ba2.so-net.ne.jp>
+;;; wl-expire.el -- Message expire modules for Wanderlust.
 
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
+;; Copyright 1998,1999,2000 Masahiro MURATA <muse@ba2.so-net.ne.jp>
+;;                          Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Masahiro MURATA <muse@ba2.so-net.ne.jp>
+;; Keywords: mail, net news
+;; Time-stamp: <00/03/14 19:34:13 teranisi>
+
+;; This file is part of Wanderlust (Yet Another Message Interface on Emacsen).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
 
 (require 'wl-summary)
 (require 'wl-thread)
 (require 'wl-folder)
+
+;;; Code:
+
 (eval-when-compile
   (require 'wl-util)
   (require 'elmo-archive))
@@ -38,42 +45,16 @@
 (defvar wl-expired-log-alist nil)
 (defvar wl-expired-log-alist-file-name "expired-log")
 
-;;; Code
-
-(defun wl-expired-alist-load (&optional file)
-  (save-excursion
-    (let ((filename (expand-file-name (or file wl-expired-alist-file-name)
-				      elmo-msgdb-dir))
-	  (tmp-buffer (get-buffer-create " *wl-expired-alist-tmp*"))
-	  insert-file-contents-pre-hook   ; To avoid autoconv-xmas...
-	  insert-file-contents-post-hook 
-	  ret-val)
-      (if (not (file-readable-p filename))
-	  ()
-	(set-buffer tmp-buffer)
-	(insert-file-contents filename)
-	(setq ret-val
-	      (condition-case nil
-		  (read (current-buffer)) 
-		(error nil nil))))
-      (kill-buffer tmp-buffer)
-      ret-val)))
-
-(defun wl-expired-alist-save (&optional file alist)
-  (save-excursion
-    (let ((filename (expand-file-name (or file wl-expired-alist-file-name)
-				      elmo-msgdb-dir))
-	  (tmp-buffer (get-buffer-create " *wl-expired-alist-tmp*")))
-      (set-buffer tmp-buffer)
-      (erase-buffer)
-      (prin1 (or alist wl-expired-alist) tmp-buffer)
-      (princ "\n" tmp-buffer)
-      (if (file-writable-p filename)
-	  (write-region (point-min) (point-max) 
-			filename nil 'no-msg)
-	(message (format "%s is not writable." filename)))
-      (kill-buffer tmp-buffer))))
+(defun wl-expired-alist-load ()
+  (elmo-object-load (expand-file-name
+		     wl-expired-alist-file-name
+		     elmo-msgdb-dir)))
 
+(defun wl-expired-alist-save (&optional alist)
+  (elmo-object-save (expand-file-name
+		     wl-expired-alist-file-name
+		     elmo-msgdb-dir)
+		    (or alist wl-expired-alist)))
 
 (defsubst wl-expire-msg-p (msg-num mark-alist)
   (cond ((consp wl-summary-expire-reserve-marks)
@@ -152,10 +133,10 @@
 		       "Expiring (move %s)"))
 	   (mess (format (concat doingmes " %s msgs...")
 			 dst-folder (length refile-list))))
-      (message mess)
+      (message "%s" mess)
       (unless (or (elmo-folder-exists-p dst-folder)
 		  (elmo-create-folder dst-folder))
-	(error "%s: create folder failed." dst-folder))
+	(error "%s: create folder failed" dst-folder))
       (if wl-expire-add-seen-list
 	  (elmo-msgdb-add-msgs-to-seen-list 
 	   dst-folder
@@ -172,7 +153,7 @@
 			  preserve-number)
 	  (progn
 	    (wl-expire-append-log folder refile-list dst-folder (if copy 'copy 'move))
-	    (message (concat mess "done")))
+	    (message "%s" (concat mess "done")))
 	(error (concat mess "failed!")))))
     (cons refile-list (length refile-list))))
 
@@ -187,12 +168,13 @@ If REFILE-LIST includes reserve mark message, so copy."
 	  (number-alist (elmo-msgdb-get-number-alist msgdb))
 	  (ret-val t)
 	  (copy-reserve-message)
-	  msg msg-id copy-len)
+	  (copy-len 0)
+	  msg msg-id)
       (message "Expiring (move %s) %s msgs..."
 	       dst-folder (length refile-list))
       (unless (or (elmo-folder-exists-p dst-folder)
 		  (elmo-create-folder dst-folder))
-	(error "%s: create folder failed." dst-folder))
+	(error "%s: create folder failed" dst-folder))
       (while (setq msg (wl-pop msglist))
 	(unless (wl-expire-msg-p msg mark-alist)
 	  (setq msg-id (cdr (assq msg number-alist)))
@@ -255,25 +237,30 @@ If REFILE-LIST includes reserve mark message, so copy."
 	 dst-folder-base dst-folder-fmt prefix)
     (cond ((eq (car spec) 'localdir)
 	   (setq dst-folder-base (concat archive-spec (nth 1 spec))))
+	  ((stringp (nth 1 spec))
+	   (setq dst-folder-base
+		 (elmo-concat-path (format "%s%s" archive-spec (car spec))
+				   (nth 1 spec))))
 	  (t
 	   (setq dst-folder-base
 		 (elmo-concat-path (format "%s%s" archive-spec (car spec))
-				   (nth 1 spec)))))
+				   (elmo-replace-msgid-as-filename 
+				    src-folder)))))
     (setq dst-folder-fmt (format fmt
 				 dst-folder-base
 				 wl-expire-archive-folder-type))
     (setq dst-folder-base (format "%s;%s"
 				  dst-folder-base
 				  wl-expire-archive-folder-type))
-    (when wl-expire-archive-folder-prefix
+    (when (and wl-expire-archive-folder-prefix
+	       (stringp (nth 1 spec)))
       (cond ((eq wl-expire-archive-folder-prefix 'short)
 	     (setq prefix (file-name-nondirectory (nth 1 spec))))
 	    (t
 	     (setq prefix (nth 1 spec))))
       (setq dst-folder-fmt (concat dst-folder-fmt ";" prefix))
       (setq dst-folder-base (concat dst-folder-base ";" prefix)))
-    (cons dst-folder-base dst-folder-fmt)
-    ))
+    (cons dst-folder-base dst-folder-fmt)))
 
 (defsubst wl-expire-archive-get-max-number (dst-folder-base &optional regexp)
   (let ((files (reverse (sort (elmo-list-folders dst-folder-base)
@@ -316,8 +303,7 @@ If REFILE-LIST includes reserve mark message, so copy."
 						 dels)))))
 	      (setq dels nil)))
 	  (list msgs dels max-num (cdr folder-info) len))
-      (list msgs dels 0 "0" 0)
-      )))
+      (list msgs dels 0 "0" 0))))
 
 (defun wl-expire-archive-number1 (folder delete-list msgdb
 					 &optional preserve-number no-delete)
@@ -526,8 +512,7 @@ Refile to archive folder followed message date."
 		    (when (or (not wl-expire-number-with-reserve-marks)
 			      (wl-expire-msg-p (car msgs) mark-alist))
 		      (setq count (1- count))))
-		  (setq msgs (cdr msgs))
-		  ))))
+		  (setq msgs (cdr msgs))))))
 	   ((eq val-type 'date)
 	    (let* ((overview (elmo-msgdb-get-overview msgdb))
 		   (key-date (elmo-date-get-offset-datevec 
@@ -593,20 +578,18 @@ Refile to archive folder followed message date."
 			     wl-expire-folder-update-msgdb))
 			   (t
 			    wl-expire-folder-update-msgdb)))
-	    wl-summary-highlight wl-auto-select-first ret-val)
+	    (wl-summary-highlight (if (or (wl-summary-sticky-p entity)
+					  (wl-summary-always-sticky-folder-p
+					   entity))
+				      wl-summary-highlight))
+	    wl-auto-select-first ret-val)
 	(save-window-excursion
 	  (save-excursion
 	    (and update-msgdb
 		 (wl-summary-goto-folder-subr entity 'force-update nil))
 	    (setq ret-val (wl-summary-expire entity (not update-msgdb)))
-	    (if (and update-msgdb
-		     wl-summary-buffer-persistent)
-		(progn
-		  (if (and wl-summary-cache-use 
-			   (or (wl-summary-message-modified-p)
-			       (wl-summary-mark-modified-p)))
-		      (wl-summary-save-view-cache))
-		  (wl-summary-msgdb-save))
+	    (if update-msgdb
+		(wl-summary-save-status 'keep)
 	      (if ret-val
 		  (wl-folder-check-entity entity))))))))))
 
@@ -696,7 +679,7 @@ Refile to archive folder followed message date."
 		       (throw 'match t))
 		     (setq alist (cdr alist)))
 		   (and (interactive-p)
-			(message "no match %s in wl-archive-alist" folder))
+			(message "No match %s in wl-archive-alist" folder))
 		   (throw 'match nil))
 		 (or (not (interactive-p))
 		     (y-or-n-p (format "Archive %s? " folder))))
@@ -706,8 +689,7 @@ Refile to archive folder followed message date."
 	(if archive-list
 	    (message "Archiving %s is done" folder)
 	  (and (interactive-p)
-	       (message "No archive"))))
-      )))
+	       (message "No archive")))))))
 
 (defun wl-folder-archive-entity (entity)
   (cond
@@ -717,8 +699,7 @@ Refile to archive folder followed message date."
 	(wl-folder-archive-entity (car flist))
 	(setq flist (cdr flist)))))
    ((stringp entity)
-    (wl-summary-archive nil entity t)
-    )))
+    (wl-summary-archive nil entity t))))
 
 ;; append log
 
@@ -727,8 +708,7 @@ Refile to archive folder followed message date."
     (save-excursion
       (let ((tmp-buf (get-buffer-create " *wl-expire work*"))
 	    (filename (expand-file-name wl-expired-log-alist-file-name
-					elmo-msgdb-dir))
-	    )
+					elmo-msgdb-dir)))
 	(set-buffer tmp-buf)
 	(erase-buffer)
 	(if dst-folder
@@ -742,7 +722,8 @@ Refile to archive folder followed message date."
 	    (write-region (point-min) (point-max) 
 			  filename t 'no-msg)
 	  (message (format "%s is not writable." filename)))
-	(kill-buffer tmp-buf)))
-    ))
+	(kill-buffer tmp-buf)))))
 
 (provide 'wl-expire)
+
+;;; wl-expire.el ends here
