@@ -1,25 +1,34 @@
-;;;
-;;; Wanderlust -- Yet Another Message Interface on Emacsen.
-;;;
-;;; Copyright (C) 1998 Yuuichi Teranishi <teranisi@gohome.org>
-;;;
-;;; Time-stamp: <99/06/28 16:22:27 teranisi>
+;;; wl-highlight.el -- Hilight modules for Wanderlust.
 
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
+;; Copyright 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, net news
+;; Time-stamp: <00/03/05 00:59:10 teranisi>
+
+;; This file is part of Wanderlust (Yet Another Message Interface on Emacsen).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
+
+;;; Code:
+;; 
 
 (if (and (featurep 'xemacs)
 	 (featurep 'dragdrop))
@@ -33,25 +42,40 @@
     (if wl-on-nemacs
 	(require 'wl-nemacs)
       (require 'wl-mule)))
-  (or (fboundp 'extent-begin-glyph)
-      (defun extent-begin-glyph (a)))
-  (or (fboundp 'delete-extent)
-      (defun delete-extent (a)))
-  (or (fboundp 'make-extent)
-      (defun make-extent (a b)))
-  (or (fboundp 'set-extent-begin-glyph)
-      (defun set-extent-begin-glyph (a b)))
-  (or (fboundp 'set-extent-end-glyph)
-      (defun set-extent-end-glyph (a b)))
-  (or (fboundp 'extent-at)
-      (defun extent-at (a b c d e)))
-  (or (fboundp 'wl-dnd-set-drop-target)
-      (defun wl-dnd-set-drop-target (a b)))
-  (or (fboundp 'wl-dnd-set-drag-starter)
-      (defun wl-dnd-set-drag-starter (a b))))
+  (defun-maybe extent-begin-glyph (a))
+  (defun-maybe delete-extent (a))
+  (defun-maybe make-extent (a b))
+  (defun-maybe set-extent-begin-glyph (a b))
+  (defun-maybe set-extent-end-glyph (a b))
+  (defun-maybe extent-at (a b c d e))
+  (defun-maybe wl-dnd-set-drop-target (a b))
+  (defun-maybe wl-dnd-set-drag-starter (a b)))
 
 (put 'wl-defface 'lisp-indent-function 'defun)
 
+(defgroup wl-faces nil
+  "Wanderlust, Faces."
+  :prefix "wl-highlight-"
+  :group 'wl-highlight
+  :group 'wl)
+
+(defgroup wl-summary-faces nil
+  "Wanderlust, Faces of summary buffer."
+  :prefix "wl-highlight-"
+  :group 'wl-highlight
+  :group 'wl-summary)
+
+(defgroup wl-folder-faces nil
+  "Wanderlust, Faces of folder buffer."
+  :prefix "wl-highlight-"
+  :group 'wl-highlight
+  :group 'wl-folder)
+
+(defgroup wl-message-faces nil
+  "Wanderlust, Faces of message buffer."
+  :prefix "wl-highlight-"
+  :group 'wl-highlight)
+
 ;; for message header and signature
 
 (wl-defface wl-highlight-message-headers
@@ -64,9 +88,10 @@
      (:foreground "gray" :bold t))
     (((class color)
       (background light))
-     (:foreground "gray50" :bold t))
-   )
-  "Face used for displaying header names.")
+     (:foreground "gray50" :bold t)))
+  "Face used for displaying header names."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-header-contents
   '(
@@ -78,9 +103,10 @@
      (:foreground "LightSkyBlue" :bold t))
     (((class color)
       (background light))
-     (:foreground "purple" :bold t))
-   )
-  "Face used for displaying header content.")
+     (:foreground "purple" :bold t)))
+  "Face used for displaying header content."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-important-header-contents
   '(
@@ -92,9 +118,10 @@
      (:foreground "yellow" :bold t))
     (((class color)
       (background light))
-     (:foreground "brown" :bold t))
-   )
-  "Face used for displaying contents of special headers.")
+     (:foreground "brown" :bold t)))
+  "Face used for displaying contents of special headers."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-important-header-contents2
   '(
@@ -106,9 +133,10 @@
      (:foreground "orange" :bold t))
     (((class color)
       (background light))
-     (:foreground "DarkSlateBlue" :bold t))
-   )
-  "Face used for displaying contents of special headers.")
+     (:foreground "DarkSlateBlue" :bold t)))
+  "Face used for displaying contents of special headers."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-citation-header
   '(
@@ -120,9 +148,10 @@
      (:foreground "SkyBlue"))
     (((class color)
       (background light))
-     (:foreground "DarkGreen"))
-   )
-  "Face used for displaying header of quoted texts.")
+     (:foreground "DarkGreen")))
+  "Face used for displaying header of quoted texts."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-unimportant-header-contents
   '(
@@ -134,9 +163,10 @@
      (:foreground "GreenYellow" :bold t))
     (((class color)
       (background light))
-     (:foreground "DarkGreen" :bold t))
-   )
-  "Face used for displaying contents of unimportant headers.")
+     (:foreground "DarkGreen" :bold t)))
+  "Face used for displaying contents of unimportant headers."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-signature
   '((((class color)
@@ -145,7 +175,9 @@
     (((class color)
       (background light))
      (:foreground "DarkSlateBlue")))
-  "Face used for displaying signature.")
+  "Face used for displaying signature."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 ;; for draft
 
@@ -155,9 +187,10 @@
       (background dark))
      (:foreground "black" :background "yellow"))
     (((class color))
-     (:foreground "Black" :background "DarkKhaki"))
-   )
-  "Face used for displaying header separator.")
+     (:foreground "Black" :background "DarkKhaki")))
+  "Face used for displaying header separator."
+  :group 'wl-draft
+  :group 'wl-faces)
 
 ;; important messages
 
@@ -171,9 +204,10 @@
      (:foreground "orange"))
     (((class color)
       (background light))
-     (:foreground "purple"))
-   )
-  "Face used for displaying important messages.")
+     (:foreground "purple")))
+  "Face used for displaying important messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-summary-new-face
   '(
@@ -185,19 +219,24 @@
      (:foreground "tomato"))
     (((class color)
       (background light))
-     (:foreground "tomato"))
-   )
-  "Face used for displaying new messages.")
+     (:foreground "tomato")))
+  "Face used for displaying new messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-summary-displaying-face
   '((t
      (:underline t :bold t)))
-  "Face used for displaying message.")
+  "Face used for displaying message."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-thread-indent-face
   '((t
      (:foreground "gray40")))
-  "Face used for displaying indented thread.")
+  "Face used for displaying indented thread."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 ;; unimportant messages
 
@@ -211,9 +250,10 @@
      (:foreground "LightSkyBlue"))
     (((class color)
       (background light))
-     (:foreground "RoyalBlue"))
-   )
-  "Face used for displaying unread messages.")
+     (:foreground "RoyalBlue")))
+  "Face used for displaying unread messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-summary-deleted-face
   '(
@@ -225,9 +265,10 @@
      (:foreground "gray"))
     (((class color)
       (background light))
-     (:foreground "DarkKhaki"))
-   )
-  "Face used for displaying messages mark as deleted.")
+     (:foreground "DarkKhaki")))
+  "Face used for displaying messages mark as deleted."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-summary-refiled-face
   '(
@@ -239,9 +280,10 @@
      (:foreground "blue"))
     (((class color)
       (background light))
-     (:foreground "firebrick"))
-   )
-  "Face used for displaying messages mark as refiled.")
+     (:foreground "firebrick")))
+  "Face used for displaying messages mark as refiled."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-summary-copied-face
   '(
@@ -253,19 +295,91 @@
      (:foreground "cyan"))
     (((class color)
       (background light))
-     (:foreground "blue"))
-   )
-  "Face used for displaying messages mark as copied.")
+     (:foreground "blue")))
+  "Face used for displaying messages mark as copied."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
+;; obsolete.
 (wl-defface wl-highlight-summary-temp-face
   '(
     (((type tty)
       (background dark))
      (:foreground "gold"))
     (((class color))
-     (:foreground "HotPink1"))
-   )
-  "Face used for displaying messages mark as temp.")
+     (:foreground "HotPink1")))
+  "Face used for displaying messages mark as temp."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-summary-target-face
+  '(
+    (((type tty)
+      (background dark))
+     (:foreground "gold"))
+    (((class color))
+     (:foreground "HotPink1")))
+  "Face used for displaying messages mark as target."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-summary-low-read-face
+  '(
+    (((type tty)
+      (background dark))
+     (:foreground "yellow" :italic t))
+    (((class color)
+      (background dark))
+     (:foreground "PaleGreen" :italic t))
+    (((class color)
+      (background light))
+     (:foreground "Green3" :italic t)))
+  "Face used for displaying low interest read messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-summary-high-read-face
+  '(
+    (((type tty))
+     (:bold t))
+    (((class color)
+      (background dark))
+     (:foreground "PaleGreen" :bold t))
+    (((class color)
+      (background light))
+     (:foreground "SeaGreen" :bold t)))
+  "Face used for displaying high interest read messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-summary-low-unread-face
+  '(
+    (((type tty)
+      (background dark))
+     (:foreground "cyan" :italic t))
+    (((class color)
+      (background dark))
+     (:foreground "LightSkyBlue" :italic t))
+    (((class color)
+      (background light))
+     (:foreground "RoyalBlue" :italic t)))
+  "Face used for displaying low interest unread messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-summary-high-unread-face
+  '(
+    (((type tty))
+     (:foreground "red" :bold t))
+    (((class color)
+      (background dark))
+     (:foreground "tomato" :bold t))
+    (((class color)
+      (background light))
+     (:foreground "tomato" :bold t)))
+  "Face used for displaying high interest unread messages."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 ;; ordinary messages
 
@@ -279,9 +393,10 @@
      (:foreground "GreenYellow"))
     (((class color)
       (background light))
-     (:foreground "green4"))
-   )
-  "Face used for displaying top thread message.")
+     (:foreground "green4")))
+  "Face used for displaying top thread message."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-summary-normal-face
   '(
@@ -293,9 +408,10 @@
      (:foreground "PaleGreen"))
     (((class color)
       (background light))
-     (:foreground "SeaGreen"))
-   )
-  "Face used for displaying normal message.")
+     (:foreground "SeaGreen")))
+  "Face used for displaying normal message."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 ;; folder
 
@@ -309,9 +425,10 @@
      (:foreground "pink"))
     (((class color)
       (background light))
-     (:foreground "RoyalBlue"))
-   )
-  "Face used for displaying unread folder.")
+     (:foreground "RoyalBlue")))
+  "Face used for displaying unread folder."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-killed-face
   '(
@@ -319,9 +436,10 @@
       (background dark))
      (:foreground "gray"))
     (((class color))
-     (:foreground "gray50"))
-   )
-  "Face used for displaying killed folder.")
+     (:foreground "gray50")))
+  "Face used for displaying killed folder."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-zero-face
   '(
@@ -333,9 +451,10 @@
      (:foreground "SkyBlue"))
     (((class color)
       (background light))
-     (:foreground "OrangeRed3"))
-   )
-  "Face used for displaying folder needs no sync.")
+     (:foreground "BlueViolet")))
+  "Face used for displaying folder needs no sync."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-few-face
   '(
@@ -347,9 +466,10 @@
      (:foreground "orange"))
     (((class color)
       (background light))
-     (:foreground "purple3"))
-   )
-  "Face used for displaying folder contains few unsync messages.")
+     (:foreground "OrangeRed3")))
+  "Face used for displaying folder contains few unsync messages."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-many-face
   '(
@@ -361,9 +481,10 @@
      (:foreground "HotPink1"))
     (((class color)
       (background light))
-     (:foreground "BlueViolet"))
-   )
-  "Face used for displaying folder contains many unsync messages.")
+     (:foreground "tomato")))
+  "Face used for displaying folder contains many unsync messages."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-unread-face
   '(
@@ -375,9 +496,10 @@
      (:foreground "gold"))
     (((class color)
       (background light))
-     (:foreground "MediumVioletRed"))
-   )
-  "Face used for displaying unread folder.")
+     (:foreground "MediumVioletRed")))
+  "Face used for displaying unread folder."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-opened-face
   '(
@@ -389,9 +511,10 @@
      (:foreground "PaleGreen"))
     (((class color)
       (background light))
-     (:foreground "ForestGreen"))
-   )
-  "Face used for displaying opened group folder.")
+     (:foreground "ForestGreen")))
+  "Face used for displaying opened group folder."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-closed-face
   '(
@@ -403,14 +526,17 @@
      (:foreground "GreenYellow"))
     (((class color)
       (background light))
-     (:foreground "DarkOliveGreen4"))
-   )
-  "Face used for displaying closed group folder.")
+     (:foreground "DarkOliveGreen4")))
+  "Face used for displaying closed group folder."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-folder-path-face
   '((t
      (:bold t :underline t)))
-  "Face used for displaying path.")
+  "Face used for displaying path."
+  :group 'wl-folder-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-demo-face
   '(
@@ -422,9 +548,23 @@
      (:foreground "GreenYellow"))
     (((class color)
       (background light))
-     (:foreground "blue2"))
-   )
-  "Face used for displaying demo.")
+     (:foreground "blue2")))
+  "Face used for displaying demo."
+  :group 'wl-faces)
+
+(wl-defface wl-highlight-logo-face
+  '(
+    (((type tty)
+      (background dark))
+     (:foreground "cyan"))
+    (((class color)
+      (background dark))
+     (:foreground "SkyBlue"))
+    (((class color)
+      (background light))
+     (:foreground "SteelBlue")))
+  "Face used for displaying demo."
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-refile-destination-face
   '((((class color)
@@ -433,7 +573,9 @@
     (((class color)
       (background light))
      (:foreground "red")))
-  "Face used for displaying refile destination.")
+  "Face used for displaying refile destination."
+  :group 'wl-summary-faces
+  :group 'wl-faces)
 
 ;; cited face
 
@@ -447,9 +589,10 @@
      (:foreground "HotPink1"))
     (((class color)
       (background light))
-     (:foreground "ForestGreen"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "ForestGreen")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-2
   '(
@@ -457,9 +600,10 @@
       (background dark))
      (:foreground "blue"))
     (((class color))
-     (:foreground "pink2"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "violet")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-3
   '(
@@ -467,9 +611,10 @@
       (background dark))
      (:foreground "cyan"))
     (((class color))
-     (:foreground "orchid3"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "orchid3")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-4
   '(
@@ -477,9 +622,10 @@
       (background dark))
      (:foreground "green"))
     (((class color))
-     (:foreground "purple1"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "purple1")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-5
   '(
@@ -487,9 +633,10 @@
       (background dark))
      (:foreground "yellow"))
     (((class color))
-     (:foreground "MediumPurple1"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "MediumPurple1")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-6
   '(
@@ -497,9 +644,10 @@
       (background dark))
      (:foreground "red"))
     (((class color))
-     (:foreground "PaleVioletRed"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "PaleVioletRed")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-7
   '(
@@ -507,9 +655,10 @@
       (background dark))
      (:foreground "magenta"))
     (((class color))
-     (:foreground "LightPink"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "LightPink")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-8
   '(
@@ -517,9 +666,10 @@
       (background dark))
      (:foreground "blue"))
     (((class color))
-     (:foreground "salmon"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "salmon")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-9
   '(
@@ -527,9 +677,10 @@
       (background dark))
      (:foreground "cyan"))
     (((class color))
-     (:foreground "SandyBrown"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "SandyBrown")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (wl-defface wl-highlight-message-cited-text-10
   '(
@@ -537,9 +688,10 @@
       (background dark))
      (:foreground "green"))
     (((class color))
-     (:foreground "wheat"))
-   )
-  "Face used for displaying quoted text from other messages.")
+     (:foreground "wheat")))
+  "Face used for displaying quoted text from other messages."
+  :group 'wl-message-faces
+  :group 'wl-faces)
 
 (defvar wl-highlight-folder-opened-regexp " *\\(\\[\\-\\]\\)")
 (defvar wl-highlight-folder-closed-regexp " *\\(\\[\\+\\]\\)")
@@ -551,7 +703,7 @@
 (defvar wl-highlight-summary-deleted-regexp " *[0-9]+D")
 (defvar wl-highlight-summary-refiled-regexp " *[0-9]+o")
 (defvar wl-highlight-summary-copied-regexp " *[0-9]+O")
-(defvar wl-highlight-summary-temp-regexp " *[0-9]+\\*")
+(defvar wl-highlight-summary-target-regexp " *[0-9]+\\*")
 ;(defvar wl-highlight-summary-thread-top-regexp " *[0-9]+[^0-9][^0-9]../..\(.*\)..:.. \\[")
 
 (defvar wl-highlight-citation-face-list
@@ -626,23 +778,37 @@
 	    (throw 'highlighted nil))
 	  (setq highlights (cdr highlights)))))))
 
-(defun-hilit2 wl-highlight-summary-line-string (line mark indent before-indent)
+(defun-hilit2 wl-highlight-summary-line-string (line mark temp-mark indent)
   (let (fsymbol)
-    (cond ((or (string= mark wl-summary-unread-cached-mark)
-	       (string= mark wl-summary-unread-uncached-mark))
-	   (setq fsymbol 'wl-highlight-summary-unread-face))
+    (cond ((and (string= temp-mark "+")
+		(member mark (list wl-summary-unread-cached-mark
+				   wl-summary-unread-uncached-mark
+				   wl-summary-new-mark)))
+	   (setq fsymbol 'wl-highlight-summary-high-unread-face))
+	  ((and (string= temp-mark "-")
+		(member mark (list wl-summary-unread-cached-mark
+				   wl-summary-unread-uncached-mark
+				   wl-summary-new-mark)))
+	   (setq fsymbol 'wl-highlight-summary-low-unread-face))
+	  ((string= temp-mark "o")
+	   (setq fsymbol 'wl-highlight-summary-refiled-face))
+	  ((string= temp-mark "O")
+	   (setq fsymbol 'wl-highlight-summary-copied-face))
+	  ((string= temp-mark "D")
+	   (setq fsymbol 'wl-highlight-summary-deleted-face))
+	  ((string= temp-mark "*")
+	   (setq fsymbol 'wl-highlight-summary-temp-face))	  
 	  ((string= mark wl-summary-new-mark)
 	   (setq fsymbol 'wl-highlight-summary-new-face))
+	  ((member mark (list wl-summary-unread-cached-mark
+			      wl-summary-unread-uncached-mark))
+	   (setq fsymbol 'wl-highlight-summary-unread-face))
 	  ((or (string= mark wl-summary-important-mark))
 	   (setq fsymbol 'wl-highlight-summary-important-face))
-	  ((string= mark "o")
-	   (setq fsymbol 'wl-highlight-summary-refiled-face))
-	  ((string= mark "O")
-	   (setq fsymbol 'wl-highlight-summary-copied-face))
-	  ((string= mark "D")
-	   (setq fsymbol 'wl-highlight-summary-deleted-face))
-	  ((string= mark "*")
-	   (setq fsymbol 'wl-highlight-summary-temp-face))
+	  ((string= temp-mark "-")
+	   (setq fsymbol 'wl-highlight-summary-low-read-face))
+	  ((string= temp-mark "+")
+	   (setq fsymbol 'wl-highlight-summary-high-read-face))
 	  (t (if (= 0 (length indent))
 		 (setq fsymbol 'wl-highlight-summary-thread-top-face)
 	       (setq fsymbol 'wl-highlight-summary-normal-face))))
@@ -687,13 +853,29 @@
 	    (setq fsymbol 'wl-highlight-summary-refiled-face)))))
       (if (not fsymbol)
 	  (cond 
-	   ((or (string= status-mark wl-summary-unread-cached-mark)
-		(string= status-mark wl-summary-unread-uncached-mark))
-	    (setq fsymbol 'wl-highlight-summary-unread-face))
+	   ((and (string= temp-mark "+")
+		 (member status-mark (list wl-summary-unread-cached-mark
+					   wl-summary-unread-uncached-mark
+					   wl-summary-new-mark)))
+	    (setq fsymbol 'wl-highlight-summary-high-unread-face))
+	   ((and (string= temp-mark "-")
+		 (member status-mark (list wl-summary-unread-cached-mark
+					   wl-summary-unread-uncached-mark
+					   wl-summary-new-mark)))
+	    (setq fsymbol 'wl-highlight-summary-low-unread-face))
 	   ((string= status-mark wl-summary-new-mark)
 	    (setq fsymbol 'wl-highlight-summary-new-face))
-	   ((or (string= status-mark wl-summary-important-mark))
+	   ((member status-mark (list wl-summary-unread-cached-mark
+				      wl-summary-unread-uncached-mark))
+	    (setq fsymbol 'wl-highlight-summary-unread-face))
+	   ((string= status-mark wl-summary-important-mark)
 	    (setq fsymbol 'wl-highlight-summary-important-face))
+	   ;; score mark
+	   ((string= temp-mark "-")
+	    (setq fsymbol 'wl-highlight-summary-low-read-face))
+	   ((string= temp-mark "+")
+	    (setq fsymbol 'wl-highlight-summary-high-read-face))
+	   ;;
 	   (t (if (and looked-at
 		       (string= (buffer-substring
 				 (match-beginning 3) 
@@ -817,7 +999,7 @@ interpreted as cited text.)"
 	      (goto-char start)
 	      (setq i 0)
 	      (while (not (eobp))
-		(wl-highlight-summary-current-line)
+		(wl-highlight-summary-current-line nil nil wl-summary-scored)
 		(setq i (+ i 1))
 		(setq percent (/ (* i 100) lines))
 		(if (eq (% percent 5) 0)
@@ -825,7 +1007,7 @@ interpreted as cited text.)"
 		(forward-line 1))
 	      (message "Highlighting...done.")))))))
 
-(defun-hilit wl-highlight-headers ()
+(defun wl-highlight-headers ()
   (let ((beg (point-min))
 	(end (or (save-excursion (re-search-forward "^$" nil t))
 		 (point-max))))
@@ -834,6 +1016,9 @@ interpreted as cited text.)"
 	 (funcall wl-highlight-x-face-func beg end))
     (run-hooks 'wl-highlight-headers-hook)))
 
+(defun wl-highlight-body-all ()
+  (wl-highlight-message (point-min) (point-max) t t))
+
 (defun-hilit wl-highlight-body ()
   (let ((beg (or (save-excursion (goto-char (point-min))
 				 (re-search-forward "^$" nil t))
@@ -844,7 +1029,6 @@ interpreted as cited text.)"
 (defun-hilit2 wl-highlight-body-region (beg end)
   (wl-highlight-message beg end t t))
 
-
 (defun wl-highlight-signature-search-simple (beg end)
   "Search signature area in the body message between beg and end.
 Returns start point of signature."
@@ -854,8 +1038,7 @@ Returns start point of signature."
 	(if (eq (char-after (point)) ?\n)
 	    (1+ (point))
 	  (point))
-      end)
-    ))
+      end)))
 
 (defun wl-highlight-signature-search (beg end)
   "Search signature area in the body message between beg and end.
@@ -967,8 +1150,7 @@ interpreted as cited text.)"
 	       (t
 		(put-text-property
 		 (match-beginning 2) (match-end 2)
-		 'face 'wl-highlight-message-header-contents))
- 	       )
+		 'face 'wl-highlight-message-header-contents)))
  	      (goto-char hend))
 	     ((looking-at mail-header-separator)
 	      (put-text-property (match-beginning 0) (match-end 0)
@@ -986,8 +1168,7 @@ interpreted as cited text.)"
 	      nil) 
 	     ((looking-at wl-highlight-force-citation-header-regexp)
 	      (setq current 'wl-highlight-message-citation-header)
-	      (setq end (match-end 0))
-	      )
+	      (setq end (match-end 0)))
 	     ((null wl-highlight-citation-prefix-regexp)
 	      nil)
 	     ((looking-at wl-highlight-citation-prefix-regexp)
@@ -1008,30 +1189,23 @@ interpreted as cited text.)"
 				       wl-highlight-citation-face-list)))))))
 	      (unless wl-highlight-highlight-citation-too
 		(goto-char (match-end 0)))
-	      (setq current (cdr pair))
-	      )
+	      (setq current (cdr pair)))
 	     ((null wl-highlight-citation-header-regexp)
 	      nil)
 	     ((looking-at wl-highlight-citation-header-regexp)
 	      (setq current 'wl-highlight-message-citation-header)
-	      (setq end (match-end 0))
-	      )
+	      (setq end (match-end 0)))
 	     (t (setq current nil)))
 	    (cond (current
 		   (setq p (point))
 		   (forward-line 1) ; this is to put the \n in the face too
-		   (let ((inhibit-read-only t))
+		   (let ();(inhibit-read-only t))
 		     (put-text-property p (or end (point))
 					'face current)
-		     (setq end nil)
-		     )
-		   (forward-char -1)
-		   ))
+		     (setq end nil))
+		   (forward-char -1)))
 	    (forward-line 1)))
-	  (run-hooks 'wl-highlight-message-hook)
-	  ))
-      )
-    ))
+	  (run-hooks 'wl-highlight-message-hook))))))
 
 
 ;; highlight-mouse-line for folder mode
@@ -1045,4 +1219,4 @@ interpreted as cited text.)"
 	 (inhibit-read-only t))
     (put-text-property beg end 'mouse-face 'highlight)))
 
-(provide 'wl-highlight)
+;;; wl-highlight.el ends here
