@@ -1,27 +1,37 @@
-;;;
-;;; Wanderlust -- Yet Another Message Interface on Emacsen.
-;;;
-;;; Copyright (C) 1998 Yuuichi Teranishi <teranisi@gohome.org>
-;;;
-;;; Time-stamp: <99/06/09 23:54:02 teranisi>
+;;; wl-thread.el -- Thread display modules for Wanderlust.
 
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;;; Boston, MA 02111-1307, USA.
-;;;
+;; Copyright 1998,1999,2000 Yuuichi Teranishi <teranisi@gohome.org>
+
+;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Keywords: mail, net news
+;; Time-stamp: <00/03/13 16:10:25 teranisi>
+
+;; This file is part of Wanderlust (Yet Another Message Interface on Emacsen).
+
+;; This program is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
+;;
+
+;;; Commentary:
+;; 
+
+;;; Code:
+;; 
 
 (require 'wl-summary)
+(require 'wl-highlight)
 
 ;; buffer local variables.
 ;(defvar wl-thread-top-entity '(nil t nil nil)) ; top entity
@@ -45,7 +55,7 @@
 ;;;;;; each entity is (number opened-or-not children parent) ;;;;;;;
 
 (defun wl-meaning-of-mark (mark)
-  (if (not wl-plugged)
+  (if (not (elmo-folder-plugged-p wl-summary-buffer-folder-name))
       (cond 
        ((string= mark wl-summary-unread-cached-mark)
 	'unread)
@@ -61,7 +71,7 @@
       'important))))
   
 (defun wl-thread-next-mark-p (mark next)
-  (cond ((not wl-plugged)
+  (cond ((not (elmo-folder-plugged-p wl-summary-buffer-folder-name))
 	 (or (string= mark wl-summary-unread-cached-mark)
 	     (string= mark wl-summary-important-mark)))
 	((eq next 'new)
@@ -77,7 +87,7 @@
 	     (string= mark wl-summary-important-mark)))))
 
 (defun wl-thread-next-failure-mark-p (mark next)
-  (cond ((not wl-plugged)
+  (cond ((not (elmo-folder-plugged-p wl-summary-buffer-folder-name))
 	 (string= mark wl-summary-unread-cached-mark))
 	((or (eq next 'new)
 	     (eq next 'unread))
@@ -99,36 +109,21 @@
     (current-buffer)
     (message "Resuming thread structure...")
     ;; set obarray value.
-    (setq wl-thread-entity-hashtb (elmo-make-hash))
+    (setq wl-thread-entity-hashtb (elmo-make-hash (* (length entities) 2)))
     (mapcar 
      '(lambda (x)
        (elmo-set-hash-val (format "#%d" (car x))
 			  x
-			  wl-thread-entity-hashtb)
-       )
+			  wl-thread-entity-hashtb))
      entities)
     ;; set buffer local variables.
     (setq wl-thread-entities entities)
     (setq wl-thread-entity-list top-list)
     (message "Resuming thread structure...done.")))
 
-;; (defun wl-thread-write-entity (entity buffer)
-;;   (let ((children (wl-thread-entity-get-children entity)))
-;;     (prin1 entity buffer)
-;;     (princ " ")
-;;     (while children
-;;       (wl-thread-write-entity 
-;;        (wl-thread-get-entity (car children)) buffer)
-;;       (princ " ")
-;;       (setq children (cdr children))
-;;       )
-;;     )
-;;   )
-
 (defun wl-thread-save-entity (dir)
   (wl-thread-save-entities dir)
-  (wl-thread-save-top-list dir)
-  )
+  (wl-thread-save-top-list dir))
 
 (defun wl-thread-save-top-list (dir)
   (let ((top-file (expand-file-name wl-thread-entity-list-file dir))
@@ -213,8 +208,7 @@
        (setq ret-val (+ (or ret-val 0) (length children)))
        (wl-push msgs msgs-stack)
        (setq msgs children))
-     (setq entity (wl-thread-get-entity (car msgs)))
-     )
+     (setq entity (wl-thread-get-entity (car msgs))))
    ret-val))
 
 (defsubst wl-thread-entity-get-descendant (entity)
@@ -230,8 +224,7 @@
        (setq ret-val (append ret-val (copy-sequence children)))
        (wl-push msgs msgs-stack)
        (setq msgs children))
-     (setq entity (wl-thread-get-entity (car msgs)))
-     )
+     (setq entity (wl-thread-get-entity (car msgs))))
    ret-val))  
 
 (defsubst wl-thread-entity-get-parent-entity (entity)
@@ -273,13 +266,11 @@ ENTITY is returned."
 
 (defun wl-thread-meaning-alist-get-result (meaning-alist)
   (let ((malist meaning-alist)
-	ret-val
-	)
+	ret-val)
     (catch 'done
       (while malist
 	(if (setq ret-val (cdr (car malist)))
-	    (throw 'done ret-val)
-	  )
+	    (throw 'done ret-val))
 	(setq malist (cdr malist))))))
 
 (defun wl-thread-entity-check-prev-mark (entity prev-marks)
@@ -310,8 +301,7 @@ ENTITY is returned."
 	(unless msgs
 	  (while (and (null msgs) msgs-stack)
 	    (setq msgs (wl-pop msgs-stack))))
-      (setq entity (wl-thread-get-entity (car msgs)))
-      ))))
+      (setq entity (wl-thread-get-entity (car msgs)))))))
 
 (defun wl-thread-entity-check-next-mark (entity next-marks)
   "Check next mark. Result is stored in NEXT-MARK."
@@ -340,8 +330,7 @@ ENTITY is returned."
 	(unless msgs
 	  (while (and (null msgs) msgs-stack)
 	    (setq msgs (wl-pop msgs-stack)))))
-      (setq entity (wl-thread-get-entity (car msgs)))
-      ))))
+      (setq entity (wl-thread-get-entity (car msgs)))))))
 
 (defun wl-thread-entity-get-older-brothers (entity &optional parent)
   (let* ((parent (or parent
@@ -423,11 +412,8 @@ ENTITY is returned."
 		     (cdr prev-marks)))
 	      marked)))))
 
-(defun wl-thread-jump-to-prev-unread (&optional hereto)
-  "If prev unread is a children of a closed message, 
-the closed parent will be opened."
-  (interactive "P")
-  (let ((cur-entity (wl-thread-get-entity (wl-summary-message-number)))
+(defun wl-thread-get-prev-unread (msg &optional hereto)
+  (let ((cur-entity (wl-thread-get-entity msg))
 	(prev-marks (cond ((eq wl-summary-move-order 'new)
 			   (cons (list (cons 'new nil))
 				 (list (cons 'unread nil)
@@ -449,7 +435,7 @@ the closed parent will be opened."
 				     (caaar prev-marks))
 	  ;;(setq mark (cons cur-entity
 	  ;;(wl-thread-entity-get-mark cur-entity)))
-	  (setq ret-val t)))
+	  (setq ret-val msg)))
     (when (and (not ret-val)
 	       (or (setq cur-entity 
 			 (wl-thread-entity-get-prev-marked-entity
@@ -459,31 +445,39 @@ the closed parent will be opened."
 	       (catch 'done
 		 (let ((success-list (car prev-marks)))
 		   (while success-list
-		     (if (not (null (cdr (car success-list))))
+		     (if (cdr (car success-list))
 			 (throw 'done nil))
 		     (setq success-list (cdr success-list)))
 		   t))
 	       (wl-thread-next-failure-mark-p mark (caaar prev-marks)))
-	  (setq ret-val t)
+	  (setq ret-val msg)
 	(when cur-entity
-	  (wl-thread-entity-force-open cur-entity)
-	  (wl-summary-jump-to-msg (car cur-entity))
-	  (setq ret-val t))))
+	  (setq ret-val (car cur-entity)))))
     ret-val))
     
+(defun wl-thread-jump-to-prev-unread (&optional hereto)
+  "If prev unread is a children of a closed message, 
+the closed parent will be opened."
+  (interactive "P")
+  (let ((msg (wl-thread-get-prev-unread
+	      (wl-summary-message-number) hereto)))
+    (when msg
+      (wl-thread-entity-force-open (wl-thread-get-entity msg))
+      (wl-summary-jump-to-msg msg)
+      t)))
+
 (defun wl-thread-jump-to-msg (&optional number)
   (interactive)
-  (let ((num (or number (read-from-minibuffer "Jump to Message(No.): "))))  
+  (let ((num (or number 
+                 (string-to-int 
+                  (read-from-minibuffer "Jump to Message(No.): ")))))
     (wl-thread-entity-force-open (wl-thread-get-entity num))
     (wl-summary-jump-to-msg num)))
 
-(defun wl-thread-jump-to-next-unread (&optional hereto)
-  "If next unread is a children of a closed message, 
-the closed parent will be opened."
-  (interactive "P")
-  (let ((cur-entity (wl-thread-get-entity
-		     (wl-summary-message-number)))
-	(next-marks (cond ((not wl-plugged)
+(defun wl-thread-get-next-unread (msg &optional hereto)
+  (let ((cur-entity (wl-thread-get-entity msg))
+	(next-marks (cond ((not (elmo-folder-plugged-p
+				 wl-summary-buffer-folder-name))
 			   (cons (list (cons 'unread nil))
 				 (list (cons 'important nil))))
 			  ((eq wl-summary-move-order 'new)
@@ -505,9 +499,7 @@ the closed parent will be opened."
 					   (wl-thread-entity-get-mark 
 					    (car cur-entity)))
 				     (caaar next-marks))
-	  ;(setq mark (cons cur-entity
-	  ;(wl-thread-entity-get-mark cur-entity)))
-	  (setq ret-val t)))
+	  (setq ret-val msg)))
     (when (and (not ret-val)
 	       (or (setq cur-entity 
 			 (wl-thread-entity-get-next-marked-entity 
@@ -518,37 +510,45 @@ the closed parent will be opened."
 	       (catch 'done
 		 (let ((success-list (car next-marks)))
 		   (while success-list
-		     (if (not (null (cdr (car success-list))))
-			 (throw 'done nil))
+		     (if (cdr (car success-list))
+		       (throw 'done nil))
 		     (setq success-list (cdr success-list)))
 		   t))
 	       (wl-thread-next-failure-mark-p mark (caaar next-marks)))
-	  (setq ret-val t)
+	  (setq ret-val msg)
 	(when cur-entity
-	  (wl-thread-entity-force-open cur-entity)
-	  (wl-summary-jump-to-msg (car cur-entity))
-	  (setq ret-val t))))
+	  (setq ret-val (car cur-entity)))))
     ret-val))
 
+(defun wl-thread-jump-to-next-unread (&optional hereto)
+  "If next unread is a children of a closed message, 
+the closed parent will be opened."
+  (interactive "P")
+  (let ((msg (wl-thread-get-next-unread
+	      (wl-summary-message-number) hereto)))
+    (when msg
+      (wl-thread-entity-force-open (wl-thread-get-entity msg))
+      (wl-summary-jump-to-msg msg)
+      t)))
+
 (defun wl-thread-close-all ()
   "Close all top threads."
   (interactive)
   (message "Closing all threads...")
-  (let ((children wl-thread-entity-list)
+  (let ((entities wl-thread-entity-list)
 	(cur 0)
 	(len (length wl-thread-entity-list)))
-    (while children
+    (while entities
       (when (and (wl-thread-entity-get-opened (wl-thread-get-entity
-					       (car children)))
+					       (car entities)))
 		 (wl-thread-entity-get-children (wl-thread-get-entity
-						 (car children))))
-	(wl-summary-jump-to-msg (car children))
+						 (car entities))))
+	(wl-summary-jump-to-msg (car entities))
 	(wl-thread-open-close)
 	(setq cur (1+ cur))
 	(message "Closing all threads...%d%%" 
-		 (/ (* cur 100) len))
-	)
-      (setq children (cdr children))))
+		 (/ (* cur 100) len)))
+      (setq entities (cdr entities))))
   (message "Closing all threads...done.")
   (goto-char (point-max)))
 
@@ -633,8 +633,7 @@ the closed parent will be opened."
   (wl-delete-all-overlays)
   (while msgs
     (setq msgs
-	  (wl-thread-update-line-on-buffer (car msgs) nil msgs))
-    ))
+	  (wl-thread-update-line-on-buffer (car msgs) nil msgs))))
 
 (defsubst wl-thread-update-line-on-buffer-sub (entity &optional msg parent-msg)
   (let ((number-alist (elmo-msgdb-get-number-alist wl-summary-buffer-msgdb))
@@ -649,12 +648,14 @@ the closed parent will be opened."
 	summary-line)
     (if (memq msg wl-summary-buffer-delete-list)
 	(setq temp-mark "D"))
-    (if (memq msg wl-summary-buffer-temp-mark-list)
+    (if (memq msg wl-summary-buffer-target-mark-list)
 	(setq temp-mark "*"))
     (if (assq msg wl-summary-buffer-refile-list)
 	(setq temp-mark "o"))
     (if (assq msg wl-summary-buffer-copy-list)
 	(setq temp-mark "O"))
+    (unless temp-mark
+      (setq temp-mark (wl-summary-get-score-mark msg)))
     ;(setq parent-entity (wl-thread-entity-get-parent-entity entity))
     (unless parent-msg
       (setq parent-msg (wl-thread-entity-get-parent entity)))
@@ -678,9 +679,8 @@ the closed parent will be opened."
 		 nil
 	       (if (not (wl-thread-entity-get-opened entity))
 		   (or children-num)))
-	     temp-mark entity
-	     ))
-      (insert summary-line"\n"))))
+	     temp-mark entity))
+      (wl-summary-insert-line summary-line))))
 
 (defun wl-thread-update-line-on-buffer (&optional msg parent-msg updates)
   (interactive)
@@ -688,7 +688,7 @@ the closed parent will be opened."
 	entity children msgs-stack)
    (while msgs
     (setq msg (wl-pop msgs))
-    (setq updates (delete msg updates))
+    (setq updates (and updates (delete msg updates)))
     (when (wl-thread-delete-line-from-buffer msg)
       (setq entity (wl-thread-get-entity msg))
       (wl-thread-update-line-on-buffer-sub entity msg parent-msg)
@@ -699,8 +699,7 @@ the closed parent will be opened."
 	  (when (wl-thread-entity-get-opened entity)
 	    (wl-push msgs msgs-stack)
 	    (setq parent-msg msg
-		  msgs children
-		  ))
+		  msgs children))
 	(unless msgs
 	  (while (and (null msgs) msgs-stack)
 	    (setq msgs (wl-pop msgs-stack)))
@@ -708,9 +707,7 @@ the closed parent will be opened."
 	    (setq parent-msg
 		  (wl-thread-entity-get-number
 		   (wl-thread-entity-get-parent-entity
-		    (wl-thread-get-entity (car msgs)))))
-	    )))
-      ))
+		    (wl-thread-get-entity (car msgs))))))))))
    updates))
 
 (defun wl-thread-delete-line-from-buffer (msg)
@@ -771,8 +768,7 @@ the closed parent will be opened."
 	  ;; top...children becomes top.
 	  (mapcar '(lambda (x)
 		    (wl-thread-entity-set-parent (wl-thread-get-entity x)
-						 nil)
-		    )
+						 nil))
 		  (setq children (wl-thread-entity-get-children entity)))
 	  ;; delete myself from top list.
 	  (setq older-brothers (wl-thread-entity-get-older-brothers
@@ -811,8 +807,7 @@ the closed parent will be opened."
  		(progn
 		  (setq num (wl-thread-entity-get-number parent))
  		  (when num
- 		    (wl-thread-update-line-on-buffer num)
- 		    ))
+ 		    (wl-thread-update-line-on-buffer num)))
  	      ;; update children lines on buffer.
  	      (mapcar '(lambda (x)
 			(wl-thread-update-line-on-buffer 
@@ -863,15 +858,15 @@ Message is inserted to the summary buffer."
 	       child-entity
 	       (elmo-msgdb-overview-get-entity-by-number overview parent-msg))
 	      (when parent
-		(wl-summary-goto-top-of-current-thread)
-		(wl-summary-message-number))) ;; return value
+		;; use thread structure.
+		(wl-thread-entity-get-number
+		 (wl-thread-entity-get-top-entity parent)))); return value;
 ;; 	      (setq beg (point))
 ;; 	      (wl-thread-goto-bottom-of-sub-thread)
 ;; 	      (wl-thread-update-indent-string-region beg (point)))
 	  ;; currently invisible.. update closed line.
-	  (wl-thread-update-children-num invisible-top)
-	  nil
-	  ))))
+	  (wl-thread-update-children-number invisible-top)
+	  nil))))
 
 (defun wl-thread-update-indent-string-thread (top-list)
   (let (beg)
@@ -882,12 +877,12 @@ Message is inserted to the summary buffer."
       (wl-thread-update-indent-string-region beg (point))
       (setq top-list (cdr top-list)))))
 
-(defun wl-thread-update-children-num (entity)
-  "update the children number."
+(defun wl-thread-update-children-number (entity)
+  "Update the children number."
   (save-excursion
     (wl-summary-jump-to-msg (wl-thread-entity-get-number entity))
     (beginning-of-line)
-    (let ((text-prop (get-text-property (point) 'font))
+    (let ((text-prop (get-text-property (point) 'face))
 	  from from-end beg str)
       (cond 
        ((looking-at (concat "^" wl-summary-buffer-number-regexp 
@@ -920,105 +915,150 @@ Message is inserted to the summary buffer."
 		    from)))
 	(if wl-summary-highlight
 	    (put-text-property 0 (length str) 'face text-prop str))
-	(insert str))))))
+	(insert str)
+	(condition-case nil ; it's dangerous, so ignore error.
+	    (run-hooks 'wl-thread-update-children-number-hook)
+	  (error
+	   (ding)
+	   (message "Error in wl-thread-update-children-number-hook."))))))))
 
 ;; 
 ;; Thread oriented commands.
 ;;
-(defun wl-thread-prefetch (&optional arg)
-  (interactive "P")
+(defun wl-thread-call-region-func (func &optional arg)
   (save-excursion
     (if arg
-	(wl-summary-goto-top-of-current-thread))
+	(wl-summary-goto-top-of-current-thread)
+      (beginning-of-line))
     (let ((beg (point)))
       (wl-thread-goto-bottom-of-sub-thread)
-      ;(end-of-line)
-      (wl-summary-prefetch-region beg (point)))))
+      (funcall func beg (point)))))
+
+(defun wl-thread-prefetch (&optional arg)
+  (interactive "P")
+  (wl-thread-call-region-func 'wl-summary-prefetch-region arg))
 
 (defun wl-thread-msg-mark-as-read (msg)
-  "Set mark as unread for invisible MSG. Modeline is not changed."
+  "Set mark as read for invisible MSG. Modeline is not changed."
   (let* ((msgdb wl-summary-buffer-msgdb)
 	 (mark-alist (elmo-msgdb-get-mark-alist msgdb))
 	 cur-mark)
     (setq cur-mark (cadr (assq msg mark-alist)))
     (cond ((or (string= cur-mark wl-summary-new-mark)
 	       (string= cur-mark wl-summary-unread-uncached-mark))
-	   ;; N,U -> u
+	   ;; N,U -> u or " "
 	   (setq mark-alist
 		 (elmo-msgdb-mark-set mark-alist
-				      msg 
-				      wl-summary-read-uncached-mark))
+				      msg
+				      (if (elmo-use-cache-p
+					   wl-summary-buffer-folder-name
+					   msg)
+					  wl-summary-read-uncached-mark)))
 	   (elmo-msgdb-set-mark-alist msgdb mark-alist)
-	   (wl-summary-set-mark-modified)
-	   )
+	   (wl-summary-set-mark-modified))
 	  ((string= cur-mark wl-summary-unread-cached-mark)
 	   ;; "!" -> " "
 	   (setq mark-alist (elmo-msgdb-mark-set mark-alist msg nil))
 	   (elmo-msgdb-set-mark-alist msgdb mark-alist)
-	   (wl-summary-set-mark-modified)
-	   ))))
+	   (wl-summary-set-mark-modified)))))
+
+(defun wl-thread-msg-mark-as-unread (msg)
+  "Set mark as unread for invisible MSG. Modeline is not changed."
+  (let* ((msgdb wl-summary-buffer-msgdb)
+	 (mark-alist (elmo-msgdb-get-mark-alist msgdb))
+	 cur-mark)
+    (setq cur-mark (cadr (assq msg mark-alist)))
+    (cond ((string= cur-mark wl-summary-read-uncached-mark)
+	   ;; u -> U
+	   (setq mark-alist
+		 (elmo-msgdb-mark-set mark-alist
+				      msg
+				      wl-summary-unread-uncached-mark))
+	   (elmo-msgdb-set-mark-alist msgdb mark-alist)
+	   (wl-summary-set-mark-modified))
+	  ((null cur-mark)
+	   ;; " " -> "!"
+	   (setq mark-alist (elmo-msgdb-mark-set mark-alist msg 
+				      wl-summary-unread-cached-mark))
+	   (elmo-msgdb-set-mark-alist msgdb mark-alist)
+	   (wl-summary-set-mark-modified)))))
+
+(defun wl-thread-msg-mark-as-important (msg)
+  "Set mark as important for invisible MSG. Modeline is not changed."
+  (let* ((msgdb wl-summary-buffer-msgdb)
+	 (mark-alist (elmo-msgdb-get-mark-alist msgdb))
+	 cur-mark)
+    (setq cur-mark (cadr (assq msg mark-alist)))
+    (setq mark-alist
+	  (elmo-msgdb-mark-set mark-alist
+			       msg
+			       (if (string= cur-mark wl-summary-important-mark)
+				   nil
+				 wl-summary-important-mark)))
+    (elmo-msgdb-set-mark-alist msgdb mark-alist)
+    (wl-summary-set-mark-modified)))
 
 (defun wl-thread-mark-as-read (&optional arg)
   (interactive "P")
-  (save-excursion
-    (if arg
-	(wl-summary-goto-top-of-current-thread))
-    (let ((beg (point)))
-      (wl-thread-goto-bottom-of-sub-thread)
-      (wl-summary-mark-as-read-region beg (point)))))
+  (wl-thread-call-region-func 'wl-summary-mark-as-read-region arg))
+
+(defun wl-thread-mark-as-unread (&optional arg)
+  (interactive "P")
+  (wl-thread-call-region-func 'wl-summary-mark-as-unread-region arg))
+
+(defun wl-thread-mark-as-important (&optional arg)
+  (interactive "P")
+  (wl-thread-call-region-func 'wl-summary-mark-as-important-region arg))
 
 (defun wl-thread-copy (&optional arg)
   (interactive "P")
-  (save-excursion
-    (if arg
-	(wl-summary-goto-top-of-current-thread))
-    (let ((beg (point)))
-      (wl-thread-goto-bottom-of-sub-thread)
-      (wl-summary-copy-region beg (point)))))
+  (wl-thread-call-region-func 'wl-summary-copy-region arg))
 
 (defun wl-thread-refile (&optional arg)
   (interactive "P")
-  (if arg
-      (wl-summary-goto-top-of-current-thread))
-  (let ((beg (point)))
-    (wl-thread-goto-bottom-of-sub-thread)
-    (save-excursion
-      (wl-summary-refile-region beg (point)))))
-
+  (condition-case err
+      (progn
+	(wl-thread-call-region-func 'wl-summary-refile-region arg)
+	(if arg
+	    (wl-summary-goto-top-of-current-thread))
+	(wl-thread-goto-bottom-of-sub-thread))
+    (error
+     (elmo-display-error err t)
+     nil)))
+	
 (defun wl-thread-delete (&optional arg)
   (interactive "P")
+  (wl-thread-call-region-func 'wl-summary-delete-region arg)
   (if arg
       (wl-summary-goto-top-of-current-thread))
-  (let ((beg (point)))
+  (if (not wl-summary-move-direction-downward)
+      (wl-summary-prev)
     (wl-thread-goto-bottom-of-sub-thread)
-    (save-excursion
-      (wl-summary-delete-region beg (point)))))
+    (if wl-summary-buffer-disp-msg
+	(wl-summary-redisplay))))
 
-(defun wl-thread-temp-mark (&optional arg)
+(defun wl-thread-target-mark (&optional arg)
   (interactive "P")
-  (save-excursion
-    (if arg
-	(wl-summary-goto-top-of-current-thread))
-    (let ((beg (point)))
-      (wl-thread-goto-bottom-of-sub-thread)
-      (wl-summary-temp-mark-region beg (point)))))
+  (wl-thread-call-region-func 'wl-summary-target-mark-region arg))
 
 (defun wl-thread-unmark (&optional arg)
   (interactive "P")
-  (save-excursion
-    (if arg
-	(wl-summary-goto-top-of-current-thread))
-    (let ((beg (point)))
-      (wl-thread-goto-bottom-of-sub-thread)
-      (wl-summary-unmark-region beg (point)))))
+  (wl-thread-call-region-func 'wl-summary-unmark-region arg))
+
+(defun wl-thread-exec (&optional arg)
+  (interactive "P")
+  (wl-thread-call-region-func 'wl-summary-exec-region arg))
+
+(defun wl-thread-save (&optional arg)
+  (interactive "P")
+  (wl-thread-call-region-func 'wl-summary-save-region arg))
 
 (defun wl-thread-force-open (&optional msg-num)
   "force open current folder"
   (if msg-num
       (wl-summary-jump-to-msg msg-num))
   (let ((wl-thread-insert-force-opened t))
-    (wl-thread-open-close)
-    ))
+    (wl-thread-open-close)))
 
 (defun wl-thread-entity-force-open (entity)
   (let ((wl-thread-insert-force-opened t)
@@ -1041,8 +1081,7 @@ Message is inserted to the summary buffer."
        0
        (wl-thread-get-entity (car elist))
        nil
-       len
-       )
+       len)
       (setq cur (1+ cur))
       (message "Inserting thread...%d%%"
 	       (/ (* cur 100) len))
@@ -1056,17 +1095,20 @@ Message is inserted to the summary buffer."
 	overview-entity
 	temp-mark
 	children-num
-	summary-line)
+	summary-line
+	score)
     (when (setq msg-num (wl-thread-entity-get-number entity))
       (unless all ; all...means no temp-mark.
 	(cond ((memq msg-num wl-summary-buffer-delete-list)
 	       (setq temp-mark "D"))
-	      ((memq msg-num wl-summary-buffer-temp-mark-list)
+	      ((memq msg-num wl-summary-buffer-target-mark-list)
 	       (setq temp-mark "*"))
 	      ((assq msg-num wl-summary-buffer-refile-list)
 	       (setq temp-mark "o"))
 	      ((assq msg-num wl-summary-buffer-copy-list)
 	       (setq temp-mark "O"))))
+      (unless temp-mark
+	(setq temp-mark (wl-summary-get-score-mark msg-num)))
       (setq children-num (wl-thread-entity-get-children-num entity))
       (setq overview-entity 
 	    (elmo-msgdb-search-overview-entity
@@ -1079,16 +1121,15 @@ Message is inserted to the summary buffer."
 	       overview-entity
 	       (assoc  ; parent-entity
 		(cdr (assq (nth 0 parent-entity)
-			    number-alist)) overview)
+			   number-alist)) overview)
 	       (1+ indent)
 	       mark-alist
 	       (if wl-thread-insert-force-opened
 		   nil
 		 (if (not (wl-thread-entity-get-opened entity))
 		     (or children-num)))
-	       temp-mark entity
-	       ))
-	(insert summary-line"\n")))))
+	       temp-mark entity))
+	(wl-summary-insert-line summary-line)))))
 
 (defun wl-thread-insert-entity (indent entity parent-entity all)
   "Insert thread entity in current buffer."
@@ -1106,18 +1147,15 @@ Message is inserted to the summary buffer."
 	    (wl-push msgs msgs-stack)
 	    (setq msgs children
 		  indent (1+ indent)
-		  parent-entity entity
-		  )))
+		  parent-entity entity)))
       (unless msgs
 	(while (and (null msgs) msgs-stack)
 	  (setq msgs (wl-pop msgs-stack))
 	  (setq indent (1- indent)))
 	(when msgs
 	  (setq entity (wl-thread-get-entity (car msgs)))
-	  (setq parent-entity (wl-thread-entity-get-parent-entity entity)))
-	)
-      (setq entity (wl-thread-get-entity (car msgs)))
-      )))
+	  (setq parent-entity (wl-thread-entity-get-parent-entity entity))))
+      (setq entity (wl-thread-get-entity (car msgs))))))
 
 (defun wl-thread-descendant-p (mynumber number)
   (let ((cur (wl-thread-get-entity number))
@@ -1192,34 +1230,36 @@ Message is inserted to the summary buffer."
 	(setq msgs children)))
     ret-val))
 
-(defun wl-thread-get-children-msgs-uncached (msg)
+(defun wl-thread-get-children-msgs-uncached (msg &optional uncached-marks)
   (let ((children-msgs (wl-thread-get-children-msgs msg))
 	(mark-alist (elmo-msgdb-get-mark-alist wl-summary-buffer-msgdb))
-	cur-mark ret-val)
+	(number-alist (elmo-msgdb-get-number-alist wl-summary-buffer-msgdb))
+	mark
+	uncached-list)
     (while children-msgs
-      (if (and (setq cur-mark (cadr (assq
-				     (car children-msgs) 
-				     mark-alist)))
-	       (or (string= cur-mark wl-summary-new-mark)
-		   (string= cur-mark wl-summary-unread-uncached-mark)
-		   (string= cur-mark wl-summary-read-uncached-mark)))
-	  (wl-append ret-val (list (car children-msgs))))
+      (if (and (not (eq msg (car children-msgs))) ; except itself
+	       (or (and uncached-marks
+			(setq mark (cadr (assq (car children-msgs)
+					       mark-alist)))
+			(member mark uncached-marks))
+		   (and (not uncached-marks)
+			(null (elmo-cache-exists-p
+			       (cdr (assq (car children-msgs)
+					  number-alist)))))))
+	  (wl-append uncached-list (list (car children-msgs))))
       (setq children-msgs (cdr children-msgs)))
-    ret-val))
+    uncached-list))
 
 (defun wl-thread-get-children-msgs-with-mark (msg mark)
   (let ((children-msgs (wl-thread-get-children-msgs msg))
 	(check-func (cond ((string= mark "o")
-			   'wl-summary-msg-marked-as-refiled
-			   )
+			   'wl-summary-msg-marked-as-refiled)
 			  ((string= mark "O")
-			   'wl-summary-msg-marked-as-copied
-			   )
+			   'wl-summary-msg-marked-as-copied)
 			  ((string= mark "D")
-			   'wl-summary-msg-marked-as-deleted
-			   )
+			   'wl-summary-msg-marked-as-deleted)
 			  ((string= mark "*")
-			   'wl-summary-msg-marked-as-temp)))
+			   'wl-summary-msg-marked-as-target)))
 	ret-val)
     (while children-msgs
       (if (funcall check-func (car children-msgs))
@@ -1245,7 +1285,6 @@ Message is inserted to the summary buffer."
 			     nil)
     (delete-char 1) ; delete '\n'
     (wl-thread-print-destination-region beg (point))))
-  
 
 (defun wl-thread-open (entity)
   (let (depth beg)
@@ -1290,11 +1329,12 @@ Message is inserted to the summary buffer."
 	  (if (wl-thread-entity-get-children entity)
 	      (wl-thread-open entity)
 	    ;; closed, and has no children, close its parent!
-	    (wl-summary-jump-to-msg 
-	     (or (wl-thread-entity-get-parent entity)
-		 (wl-thread-entity-get-number entity)))
-	    (wl-thread-close
-	     (wl-thread-get-entity (wl-summary-message-number))))))  
+	    (setq msg (or (wl-thread-entity-get-parent entity)
+			  (wl-thread-entity-get-number entity)))
+	    (when msg
+	      (wl-summary-jump-to-msg msg)
+	      (wl-thread-close
+	       (wl-thread-get-entity (wl-summary-message-number)))))))
       (wl-summary-set-message-modified)
       (set-buffer-modified-p nil))))
   
@@ -1380,3 +1420,5 @@ Message is inserted to the summary buffer."
 	    (wl-highlight-summary-current-line))))))
 
 (provide 'wl-thread)
+
+;;; wl-thread.el ends here
